#!/bin/bash


###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

cd $(dirname $0)
progdir=$(basename ${PWD})
CWD=${PWD}
PKGDIR=${CWD}/pkg

# This have to be set in command line
TARGET=""
SOURCES=""
#[ x"$CC" == x ] && CC=${CROSS_COMPILE}gcc

# Set this here
CONTENTS=""
INSTALL_PREFIX="/"

export WITH_METADATA=no
export WITH_METADATA_HTML=no
export WITH_METADATA_PDF=no
###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progdir}/${progname} === $*"
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname -t TARGET -s SOURCES"
	exit 1;
}

###############################################################################
# Make all
###############################################################################
makeAll()
{
	[ x"$archm" == xe2k ] && OPT_BUILD="--build=e2k"
	if [ x"$ARCH" != x ]; then
		OPT_HOST="--host=$ARCH"
	fi

	[ x"${CC}" != x ] && OPT_CC="CC=$CC"
	[ x"${CPP}" != x ] && OPT_CPP="CPP=$CPP"

	CFLAGS="$CFLAGS -Wall -O0"
	[ x"${ROOT_WA}" != x ] && CFLAGS="${CFLAGS} -I/${ROOT_WA}/usr/include"
	[ x"${CROSSROOT}" != x ] && CFLAGS="${CFLAGS} -I${CROSSROOT}/usr/include"
	[ -e ${CROSSROOT}/usr/include/tirpc ] && CFLAGS="${CFLAGS} -I${CROSSROOT}/usr/include/tirpc"
#	[ x"${CFLAGS}" != x ] && OPT_CFLAGS="CFLAGS=$CFLAGS"

#	LDFLAGS="$LDFLAGS"
#	[ x"${CROSSROOT}" != x ] && LDFLAGS="-Wl,-rpath-link,$CROSSROOT/usr/lib $LDFLAGS"
#	if [ x"$ELDEBIAN" == x0311 ]; then
#		[ x"${CROSS_ROOT}" != x ] && LDFLAGS="-L$CWD/${SOURCES}/lib -Wl,-rpath-link,$CWD/${SOURCES}/lib $LDFLAGS"
#	fi
#	[ x"${LDFLAGS}" != x ] && OPT_LDFLAGS="LDFLAGS=$LDFLAGS"

	CPPFLAGS="$CPPFLAGS"
#	[ x"${CPPFLAGS}" != x ] && OPT_CPPFLAGS="CPPFLAGS=$CPPFLAGS"

	CFLAGS="$CFLAGS -I$CROSS_ROOT/usr/include/tirpc"

	if [ x${BUILD128} == xy ]; then
		CFLAGS="-mptr128 -mprot-all -I${CROSS_ROOT}/usr/include/${ARCH}-linux-gnu $CFLAGS"
		LDFLAGS="-mptr128 -L${CROSS_ROOT}/usr/lib/e2k128-linux-gnu $LDFLAGS"
	fi

	export CC CPP LD CFLAGS LDFLAGS CPPFLAGS

	cd ${SOURCES}
	
	echo "make autotools"
	make autotools || exit 1

	if [ x"$ELDEBIAN" == x0311 ]; then
		cp /control/CONFIG/config.guess . || exit 1
		cp /control/CONFIG/config.sub . || exit 1
	fi

	if [ x${BUILD128} == xy ]; then
		opt_metadata="--disable-metadata"
	fi

	[ x"$BUILD128" == xy ] && opt_tirpc='--without-tirpc'

#	echo "./configure $OPT_HOST $OPT_BUILD --prefix=$INSTALL_PREFIX \"$OPT_CC\" \"$OPT_CPP\" \"$OPT_CFLAGS\" \"$OPT_CPPFLAGS\" \"$OPT_LDFLAGS\""
#	./configure $OPT_HOST $OPT_BUILD --prefix=$INSTALL_PREFIX "$OPT_CC" "$OPT_CPP" "$OPT_CFLAGS" "$OPT_CPPFLAGS" "$OPT_LDFLAGS" || exit 1
	echo "./configure $OPT_HOST $OPT_BUILD --prefix=$INSTALL_PREFIX ${opt_metadata} ${opt_tirpc}"
	./configure $OPT_HOST $OPT_BUILD --prefix=$INSTALL_PREFIX ${opt_metadata} ${opt_tirpc} || exit 1
	make V=1 || exit 1
	cd ${CWD}

	cd conf
	cp ./configure.sh ./configure
	chmod a+x ./configure
	cd ${CWD}
}


###############################################################################
# Make clean
###############################################################################
makeClean()
{
	cd ${SOURCES}
	make clean
	cd ${CWD}
#return
	CONTENTS="contents/${SOURCES}-contents"
	while read FS_OBJ; do
		grep -q "^${FS_OBJ}$" ${CONTENTS}
		if [ $? -ne 0 ]; then
			rm -rf ${FS_OBJ}
		fi
	done < <(find ${SOURCES} -name "*" | grep -v "\.svn")
}


###############################################################################
# Install to directory
###############################################################################
installToDir()
{
	INSTDIR=$1
	if [ ! -d "${INSTDIR}" ]; then
		doEcho "Directory <${INSTDIR}> does not exist"
		return 1
	fi
	cd ${SOURCES}
	make install DESTDIR=${INSTDIR} CC=${CC} || exit 1

	cd ${CWD}
}


###############################################################################
# Make install
###############################################################################
makeInstall()
{
	instdir="${CWD}/root"
	mkdir -p ${instdir}
	installToDir ${instdir}

	cd ${CWD}/root/testcases/bin
	for fso in $(ls); 
	do
		[ "${fso}" == "acct02_helper" ] && continue	
		[ "${fso}" == "creat07_child" ] && continue	
		[ "${fso}" == "execve_child" ] && continue	
		[ "${fso}" == "execveat_child" ] && continue	
		[ "${fso}" == "execveat_errno" ] && continue	
		[ "${fso}" == "getrusage03_child" ] && continue	
		[ "${fso}" == "fanotify_child" ] && continue	
		[ "${fso}" == "pipe2_02_child" ] && continue	
		[ "${fso}" == "execlp01_child" ] && continue	
		[ "${fso}" == "execvp01_child" ] && continue	
		[ "${fso}" == "open12_child" ] && continue	
		[ "${fso}" == "openat02_child" ] && continue	
		[ "${fso}" == "dirtyc0w_child" ] && continue	
		[ "${fso}" == "setpgid03_child" ] && continue	
		[ "${fso}" == "syslog-lib.sh" ] && continue	
		[ "${fso}" == "cmdlib.sh" ] && continue	
		[ "${fso}" == "daemonlib.sh" ] && continue	
		[ "${fso}" == "execl01_child" ] && continue	
		[ "${fso}" == "execle01_child" ] && continue	
		[ "${fso}" == "execv01_child" ] && continue	
		[ "${fso}" == "execve01_child" ] && continue	
		[ "${fso}" == "mount03_suid_child" ] && continue	
		[ "${fso}" == "prctl06_execve" ] && continue	
		[ "${fso}" == "dirtyc0w_shmem_child" ] && continue	
		[ "${fso}" == "execve06_child" ] && continue	
		[ "${fso}" == "mmstress_dummy" ] && continue	
		[ "${fso}" == "vma05_vdso" ] && continue	
		[ "${fso}" == "pthserv" ] && continue	
		[ "${fso}" == "pthcli" ] && continue	

		[[ "${fso}" =~ ^tst_ ]] && continue	

		[[ "${fso}" =~ ^gen ]] && continue	

		grep -q "${fso}" ../../runtest/nrp && continue 
		grep -q "${fso}" ../../runtest/ipc && continue 
		grep -q "${fso}" ../../runtest/math && continue 
		grep -q "${fso}" ../../runtest/syscalls && continue 
		grep -q "${fso}" ../../runtest/mm && continue 
		grep -q "${fso}" ../../runtest/sched && continue 
		grep -q "${fso}" ../../runtest/hugetlb && continue 

		rm -rf ${fso}
	done
}


###############################################################################
# Make package
###############################################################################
makePackage()
{
	exit 1
}


###############################################################################
# Make uninstall
###############################################################################
makeUninstall()
{
	rm -rf "${CWD}/root"
	rm -f ./conf/configure
}


###############################################################################
# Make clobber
###############################################################################
makeClobber()
{
	rm -f ./*~
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
for param in "$@"
do
	case "${param}" in
	CC=*)
		p=${param}; p=${p#CC=}; p=${p#\"}; p=${p%\"}; CC=$p
	;;
	*)
			
	;;
	esac
done

while getopts ":t:s:" optname
do
	case "$optname" in
	"t")
		TARGET=$OPTARG
	;;
	"s")
		SOURCES=$OPTARG
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# Check input parameters
###############################################################################
[ -z "${TARGET}" -o -z "${SOURCES}" ] && usage

if [ ! -d "${SOURCES}" ]; then
	doEcho "Directory <${SOURCES}> does not exist"
	exit 1
fi

archm=`uname -m | sed -e s/i.86/i386/ -e s/sun4u/sparc64/`

VERSION=${SOURCES#ltp-}

###############################################################################
# Print settings
###############################################################################
doEcho "VERSION = ${VERSION}"
doEcho "ARCH = ${ARCH}"
doEcho "TARGET = ${TARGET}"
doEcho "CC = ${CC}"


###############################################################################
# Switch target
###############################################################################
case "${TARGET}" in
	"all"|"a")
		makeAll
	;;
	"install"|"i")
		makeInstall
	;;
	"package"|"pkg")
		makePackage
	;;
	"clean"|"c")
		makeClean
	;;
	"uninstall"|"u")
		makeUninstall
	;;
	"clobber"|"cl")
		makeClobber
	;;
	*)
		doEcho "Unknown target <${TARGET}>"	
	;;
esac
