#!/bin/bash

SCENS="nrp ipc math syscalls mm sched hugetlb"

CWD=$(pwd)

cd $(dirname $0)

# Create cp-copies for scens
for scen in $SCENS; 
do
	su_osltc "rm" "-f" "../root/runtest/${scen}.cp";
	cp "../root/runtest/${scen}" "../root/runtest/${scen}.cp";
done

# Remove from cp-scens unsupported tags
for confd in $(find . -name "conf_*" -type d);
do
	tag=${confd#*conf_}
	rm -rf /tmp/test_$tag
	mkdir -p /tmp/test_$tag
	cp -r "$confd" "/tmp/test_$tag/conf"
	echo "pass" > /tmp/test_$tag/scen.cp
	echo "fail" > /tmp/test_$tag/scen.cf
	conf_osltc -d /tmp/test_$tag
	res=$(cat /tmp/test_$tag/scen)
	if [ "$res" == "fail" ];
	then
		for scen in $SCENS;
		do
			scen="$scen.cp"
			cat "../root/runtest/$scen" | grep -v "^$tag" > /tmp/$scen
			cp "/tmp/$scen" "../root/runtest/$scen"
		done
	fi
done

if [[ "$(get_stress_level_osltc)" -lt "3" ]];
then
	echo -e "root:	./run.sl2" > ../scen.cp
else
	echo -e "root:	root/runltp -f nrp.cp -o nrp.root.out -l nrp.root.log" > ../scen.cp
	echo -e "root:	root/runltp -f ipc.cp -o ipc.root.out -l ipc.root.log -Q" >> ../scen.cp
	echo -e "root:	root/runltp -f math.cp -o math.root.out -l math.root.log -Q" >> ../scen.cp
	echo -e "root:	root/runltp -f syscalls.cp -o syscalls.root.out -l syscalls.root.log" >> ../scen.cp
	echo -e "root:	root/runltp -f mm.cp -o mm.root.out -l mm.root.log" >> ../scen.cp
	echo -e "root:	root/runltp -f sched.cp -o sched.root.out -l sched.root.log" >> ../scen.cp
	echo -e "root:	root/runltp -f hugetlb.cp -o hugetlb.root.out -l hugetlb.root.log" >> ../scen.cp
fi

PATH=/sbin:$PATH
MODPROBE="$(which modprobe)"

# Setup modules
su_osltc "$MODPROBE" "tun"
su_osltc "$MODPROBE" "loop"

# Cleanup logs
su_osltc "rm" "-rf" "../root/results" "../root/output"

# Cleanup /tmp
su_osltc "rm" "-rf" "/tmp/cgroup_cst"

# Unset real-time priority
ulimit -r 0

su_osltc "killall" "gvfsd" "gvfs-udisks2-volume-monitor" "gvfsd-trash"
export LTP_TIMEOUT_MUL=12

cd $CWD

exit 0

