#!/bin/bash
##############################################################################
# Copyright (c) 2021 MCST. All Rights Reserved.                              #
#                                                                            #
# This program is free software;  you can redistribute it and#or modify      #
# it under the terms of the GNU General Public License as published by       #
# the Free Software Foundation; either version 2 of the License, or          #
# (at your option) any later version.                                        #
#                                                                            #
# This program is distributed in the hope that it will be useful, but        #
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY #
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License   #
# for more details.                                                          #
#                                                                            #
# You should have received a copy of the GNU General Public License          #
# along with this program;  if not, write to the Free Software Foundation,   #
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA           #
#                                                                            #
# TEST DESCRIPTION : Test is intended to verify btrfs ioctl commands.        #
#                                                                            #
# Check list:                                                                #
#  1. BTRFS_IOC_SCAN_DEV                      +                              #
#  2. BTRFS_IOC_TREE_SEARCH                   +                              #
#  3. BTRFS_IOC_FS_INFO                       +                              #
#  4. BTRFS_IOC_DEV_INFO                      +                              #
#  5. BTRFS_IOC_SPACE_INFO                    +                              #
#  6. BTRFS_IOC_GET_FEATURES                  +                              #
#  7. BTRFS_IOC_SYNC                          +                              #
#  8. BTRFS_IOC_RESIZE                        +                              #
#  9. BTRFS_IOC_GET_FSLABEL                   +                              #
# 10. BTRFS_IOC_SET_FSLABEL                   +                              #
# 11. BTRFS_IOC_QUOTA_CTL                     +                              #
# 12. BTRFS_IOC_QUOTA_RESCAN                  +                              #
# 13. BTRFS_IOC_QUOTA_RESCAN_WAIT             +                              #
# 14. BTRFS_IOC_QUOTA_RESCAN_STATUS           +                              #
# 15. BTRFS_IOC_SUBVOL_CREATE                 +                              #
# 16. BTRFS_IOC_INO_LOOKUP                    +                              #
# 17. BTRFS_IOC_SNAP_DESTROY                  +                              #
# 18. BTRFS_IOC_GET_DEV_STATS                 +                              #
# 19. BTRFS_IOC_SCRUB                         +                              #
# 20. BTRFS_IOC_SCRUB_PROGRESS                +                              #
# 21. BTRFS_IOC_SCRUB_CANCEL                  +                              #
# 22. BTRFS_IOC_QGROUP_CREATE                 +                              #
# 23. BTRFS_IOC_QGROUP_LIMIT                  +                              #
# 24. BTRFS_IOC_QGROUP_ASSIGN                 +                              #
# 25. BTRFS_IOC_SNAP_CREATE                   (not used in btrfs tools)      #
# 26. BTRFS_IOC_DEFRAG                        (not used in btrfs tools)      #
# 27. BTRFS_IOC_CLONE                         (not used in btrfs tools)      #
# 28. BTRFS_IOC_ADD_DEV                       +                              #
# 29. BTRFS_IOC_RM_DEV                        +                              #
# 30. BTRFS_IOC_BALANCE                       (not used in btrfs tools)      #
# 31. BTRFS_IOC_CLONE_RANGE                   (not used in btrfs tools)      #
# 32. BTRFS_IOC_DEFRAG_RANGE                  +                              #
# 33. BTRFS_IOC_START_SYNC                    +                              #
# 34. BTRFS_IOC_WAIT_SYNC                     +                              #
# 35. BTRFS_IOC_SUBVOL_GETFLAGS               +                              #
# 36. BTRFS_IOC_SUBVOL_SETFLAGS               +                              #
# 37. BTRFS_IOC_BALANCE_CTL                   +                              #
# 38. BTRFS_IOC_BALANCE_PROGRESS              +                              #
# 39. BTRFS_IOC_INO_PATHS                     +                              #
# 40. BTRFS_IOC_LOGICAL_INO                   +                              #
# 41. BTRFS_IOC_SET_RECEIVED_SUBVOL           +                              #
# 42. BTRFS_IOC_SEND                          +                              #
# 43. BTRFS_IOC_DEVICES_READY                 +                              #
# 44. BTRFS_IOC_DEV_REPLACE                   +                              #
# 45. BTRFS_IOC_FILE_EXTENT_SAME              (not used in btrfs tools)      #
# 46. BTRFS_IOC_SET_FEATURES                  (not used in btrfs tools)      #
# 47. BTRFS_IOC_GET_SUPPORTED_FEATURES        (not used in btrfs tools)      #
# 48. BTRFS_IOC_DEFAULT_SUBVOL                +                              #
# 49. BTRFS_IOC_BALANCE_V2                    +                              #
# 50. BTRFS_IOC_TREE_SEARCH_V2                (not used in btrfs tools)      #
# 51. BTRFS_IOC_SUBVOL_CREATE_V2              +                              #
# 52. BTRFS_IOC_SNAP_CREATE_V2                +                              #
# 53. BTRFS_IOC_RM_DEV_V2                     +                              #
# 54. BTRFS_IOC_LOGICAL_INO_V2                +                              #
##############################################################################

TCID=ioctl_btrfs
export TST_TOTAL=1
export TST_COUNT=1

. test.sh

BTRFS=btrfs128
MKFS=mkbtrfs128
IMAGE=./file.img
LOOP_DEV=undef
IMAGE2=./file2.img
LOOP_DEV2=undef
image2_ok=1
MOUNTPOINT=./MNT
MYDIR=dir
MYSAVEDDIR=saved
LTPBIN=${LTPROOT}/testcases/bin
BTRFS_CONTROL_DEV=/dev/btrfs-control
BTRFS_CONTROL_DEV_MAJ=10
BTRFS_CONTROL_DEV_MIN=234
TST_PASSED=0
TST_FAILED=0

setup()
{
	tst_require_root
	tst_tmpdir

	if [[ ! -d ${LTPBIN} ]] ; then
		tst_brkm TCONF "Directory ${LTPBIN} not found."
	fi

	if [[ ! -e ${LTPBIN}/${BTRFS} ]] ; then
		tst_resm TINFO "Test ${BTRFS} not applicable. Skipped."
		exit 32
	fi

	log="$(${LTPBIN}/${BTRFS} test 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "Test ${BTRFS} not applicable. Skipped."
		exit 32
	fi

	# Check and load loop module
	lsmod | grep loop >/dev/null
        if [[ $? -ne 0 ]]; then
		modprobe loop
		if [[ $? -ne 0 ]]; then
			tst_brkm TCONF "modprobe loop failed"
		fi
		sleep 1
	fi

	# Check and load btrfs module
	lsmod | grep btrfs >/dev/null
        if [[ $? -ne 0 ]]; then
		modprobe btrfs
		if [[ $? -ne 0 ]]; then
			tst_brkm TCONF "modprobe btrfs failed"
		fi
		sleep 1
	fi

	# Create btrfs control device if no udev action in 3 sec.
	sleep 3
	if [[ ! -c ${BTRFS_CONTROL_DEV} ]] ; then 
		mknod -m 0660 ${BTRFS_CONTROL_DEV} c ${BTRFS_CONTROL_DEV_MAJ} ${BTRFS_CONTROL_DEV_MIN}
		if [[ $? -ne 0 ]]; then
			tst_brkm TBROK "Can not create ${BTRFS_CONTROL_DEV_DEV}."
		fi
	fi

	# Check and make a symbolic link mkbtrfs128 -> btrfs128.
	if [[ ! -L ${LTPBIN}/${MKFS} ]] ; then 
		ln -s ${LTPBIN}/${BTRFS} ${LTPBIN}/${MKFS}
		if [[ $? -ne 0 ]]; then
			tst_brkm TBROK "Can not create a link ${MKFS}."
		fi
	fi

	# Create 300MB image file.
	dd if=/dev/zero of=${IMAGE} bs=1024k count=300 >/dev/null 2>&1
	if [[ $? -ne 0 ]]; then
		tst_brkm TBROK "dd can not create 300MB image file."
	fi
	# Create 300MB image2 file.
	dd if=/dev/zero of=${IMAGE2} bs=1024k count=300 >/dev/null 2>&1
	if [[ $? -ne 0 ]]; then
		image2_ok=0
		tst_resm TINFO "dd can not create 300MB image2 file."
	fi

	sleep 1
	# Find available loop device and attach it to the image file.
	losetup -f ${IMAGE} >/dev/null 2>&1
	if [[ $? -ne 0 ]]; then
		tst_brkm TBROK "losetup --find failed."
	fi

	LOOP_DEV="$(losetup -j ${IMAGE} | cut -d : -f 1)"
	if [[ "${LOOP_DEV}" =~ ^/dev/loop[[:digit:]]+ ]] ; then
		tst_resm TINFO "Loop device attached to image file."
	else
		tst_brkm TBROK "No available loop device."
	fi

	if [[ "${image2_ok}" -eq 1 ]] ; then
		# Find available loop device and attach it to the image2 file.
		losetup -f ${IMAGE2} >/dev/null 2>&1
		if [[ $? -ne 0 ]] ; then
			tst_brkm TBROK "losetup --find failed."
			image2_ok=0
		fi

		LOOP_DEV2="$(losetup -j ${IMAGE2} | cut -d : -f 1)"
		if [[ "${LOOP_DEV2}" =~ ^/dev/loop[[:digit:]]+ ]] ; then
			tst_resm TINFO "Loop device attached to image2 file."
		else
			tst_brkm TBROK "No available loop device."
			image2_ok=0
		fi
	fi
	# Make MOUNTPOINT directory.
	mkdir ${MOUNTPOINT} >/dev/null 2>&1
	if [[ $? -ne 0 ]]; then
		tst_brkm TBROK "mkdir failed."
	fi
	# Relax for 1 sec...
	sleep 1
}

cleanup()
{
	tst_umount ${MOUNTPOINT}
	if [[ "${LOOP_DEV}" =~ ^/dev/loop[[:digit:]]+ ]] ; then
		losetup -d ${LOOP_DEV}
	fi
	if [[ "${LOOP_DEV2}" =~ ^/dev/loop[[:digit:]]+ ]] ; then
		losetup -d ${LOOP_DEV2}
	fi
	tst_rmdir
}

check_ioctl_cmds()
{
	if [[ $# -eq 0 ]] ; then
		return
	fi
	out="$1"
	while IFS=":" read mark cmd ret eno ; do
		if [[ ${mark} =~ ^IOCTL_SYSCALL ]] ; then
			if [[ "${ret}" -eq 0 ]] ; then
				TST_PASSED=$((TST_PASSED+1))
				tst_resm TPASS "${cmd}: return: 0 passed"
			else
				case ${eno} in
					2)	# ENOENT: No such file or directory - ignore! 
						TST_PASSED=$((TST_PASSED+1))
					 	tst_resm TPASS "${cmd}: return: ${eno} failed but OK"
						continue
						;;
					16)	# EBUSY:  Device or resource busy  - ignore!
						TST_PASSED=$((TST_PASSED+1))
					 	tst_resm TPASS "${cmd}: return: ${eno} failed but OK"
						continue
						;;
					19)	# ENODEV: No device - ignore! 
						TST_PASSED=$((TST_PASSED+1))
					 	tst_resm TPASS "${cmd}: return: ${eno} failed but OK"
						continue
						;;
					107)	# ENOTCONN: Kernel task already finished - ignore! 
						TST_PASSED=$((TST_PASSED+1))
					 	tst_resm TPASS "${cmd}: return: ${eno} failed but OK"
						continue
						;;
					115)	# EINPROGRESS: Operation now in progress - No problem! 
						TST_PASSED=$((TST_PASSED+1))
					 	tst_resm TPASS "${cmd}: return: ${eno} failed but OK"
						continue
						;;
					*) 	tst_resm TFAIL "${cmd}: return: ${eno} failed"
						TST_FAILED=$((TST_FAILED+1))
						;;
				esac
			fi
		fi
	done < <(echo "$out")
	return
}

get_ID()
{
	if [[ $# -eq 0 ]] ; then
		return
	fi
	out="$1"
	while IFS=" " read mark id other ; do
		if [[ ${mark} =~ ^ID$ && ${id} =~ ^[[:digit:]]+$ ]] ; then
			echo "${id}"
			return
		fi
	done < <(echo "$out")
	echo "undef"

}

do_test()
{
	local log
	# 1.
	tst_resm TINFO "Check ioctl commands on mkfs.btrfs: "
	log="$(${MKFS} -b 256M ${LOOP_DEV} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		check_ioctl_cmds "${log}"
		tst_brkm TBROK "mkfs.btrfs failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	tst_resm TINFO "Run ${BTRFS} check: "
	log="$(${BTRFS} check ${LOOP_DEV} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} check failed: ${log}"
	fi

	# 2.
	tst_resm TINFO "Check ioctl commands on btrfs device ready: "
	log="$(${BTRFS} device ready ${LOOP_DEV} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 3.
	log="$(mount ${LOOP_DEV} ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_brkm TBROK "mount failed: ${log}"
	fi

	mkdir ${MOUNTPOINT}/${MYDIR}
	mkdir ${MOUNTPOINT}/${MYSAVEDDIR}

	# 4.
	tst_resm TINFO "Check ioctl commands on btrfs fs usage: "
	log="$(${BTRFS} filesystem usage ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# Create a file to trigger sync ON.
	echo "12345678901234567890" >${MOUNTPOINT}/file1
	# 5.
	tst_resm TINFO "Check ioctl commands on btrfs fs sync: "
	log="$(${BTRFS} filesystem sync ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# Create 10MB file to trigger defragmentation ON.
	dd if=/dev/zero of=${MOUNTPOINT}/file2 bs=1024k count=10 >/dev/null 2>&1
	# 6.
	tst_resm TINFO "Check ioctl commands on btrfs fs defragment: "
	log="$(${BTRFS} filesystem defragment -f -r ${MOUNTPOINT}/* 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 7.
	tst_resm TINFO "Check ioctl commands on btrfs fs resize: "
	curr_dir="$(pwd)"
	log="$(${BTRFS} filesystem resize max ${curr_dir}/${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 8.
	tst_resm TINFO "Check ioctl commands on btrfs fs label: "
	log="$(${BTRFS} filesystem label ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 9.
	tst_resm TINFO "Check ioctl commands on btrfs fs new_label: "
	log="$(${BTRFS} filesystem label ${MOUNTPOINT} DATA  2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 10.
	tst_resm TINFO "Check ioctl commands on btrfs property list: "
	log="$(${BTRFS} property list ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 11.
	tst_resm TINFO "Check ioctl commands on btrfs quota enable: "
	log="$(${BTRFS} quota enable ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 12.
	tst_resm TINFO "Check ioctl commands on btrfs quota rescan: "
	log="$(${BTRFS} quota rescan ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 13.
	tst_resm TINFO "Check ioctl commands on btrfs quota rescan -s: "
	log="$(${BTRFS} quota rescan -s ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 14.
	tst_resm TINFO "Check ioctl commands on btrfs quota rescan -w:"
	log="$(${BTRFS} quota rescan -w ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 15.
	tst_resm TINFO "Check ioctl commands on btrfs quota disable: "
	log="$(${BTRFS} quota disable ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 16.
	tst_resm TINFO "Check ioctl commands on btrfs subvolume create:"
	log="$(${BTRFS} subvolume create ${MOUNTPOINT}/${MYDIR}/d 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 17.
	tst_resm TINFO "Check ioctl commands on btrfs subvolume delete:"
	log="$(${BTRFS} subvolume delete -C ${MOUNTPOINT}/${MYDIR}/d 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 18.
	tst_resm TINFO "Check ioctl commands on btrfs device stats:"
	log="$(${BTRFS} device stats ${LOOP_DEV} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 19.
	tst_resm TINFO "Check ioctl commands on btrfs scrub start:"
	log="$(${BTRFS} scrub start ${LOOP_DEV} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 20.
	tst_resm TINFO "Check ioctl commands on btrfs scrub cancel:"
	log="$(${BTRFS} scrub cancel ${LOOP_DEV} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed as expected"
	fi
	check_ioctl_cmds "${log}"

	# 1. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs quota enable: "
	log="$(${BTRFS} quota enable ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 2. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs subvolume create:"
	log="$(${BTRFS} subvolume create ${MOUNTPOINT}/${MYDIR}/a 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 3. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs subvolume list:"
	log="$(${BTRFS} subvolume list ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	ID="$(get_ID "${log}")"

	# 4. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs qgroup create:"
	log="$(${BTRFS} qgroup create 1/100 ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 5. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs qgroup limit:"
	log="$(${BTRFS} qgroup limit 10M 1/100 ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 6. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs qgroup assign:"
	log="$(${BTRFS} qgroup assign 0/${ID} 1/100 ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 6a. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs qgroup show:"
	log="$(${BTRFS} qgroup show --sync ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 2a. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs subvolume create -i:"
	log="$(${BTRFS} subvolume create -i 1/100 ${MOUNTPOINT}/${MYDIR}/b 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 7. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs qgroup remove:"
	log="$(${BTRFS} qgroup remove 0/${ID} 1/100 ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	sleep 2
	# 8. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs qgroup destroy:"
	log="$(${BTRFS} qgroup destroy 1/100 ${MOUNTPOINT}/${MYDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 9. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs property set:"
	log="$(${BTRFS} property set ${MOUNTPOINT}/${MYDIR}/a ro false 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 10. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs inspect-internal logical-resolve:"
	log="$(${BTRFS} inspect-internal logical-resolve 1 ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed as expected"
	fi
	check_ioctl_cmds "${log}"

	# 10a. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs inspect-internal logical-resolve -o:"
	log="$(${BTRFS} inspect-internal logical-resolve -o 1 ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed as expected"
	fi
	check_ioctl_cmds "${log}"

	# 11. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs inspect-internal inode-resolve:"
	log="$(${BTRFS} inspect-internal inode-resolve 1 ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed as expected"
	fi
	check_ioctl_cmds "${log}"

	# 12. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs subvolume set-default:"
	log="$(${BTRFS} subvolume set-default ${ID} ${MOUNTPOINT}/${MYDIR}/a 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 13. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs subvolume snapshot -r:"
	touch ${MOUNTPOINT}/${MYDIR}/a/file
	log="$(${BTRFS} subvolume snapshot -r ${MOUNTPOINT}/${MYDIR}/a ${MOUNTPOINT}/${MYDIR}/snap 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"

	# 13a. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs send:"
	log="$(${BTRFS} send -q -f ./log1 ${MOUNTPOINT}/${MYDIR}/snap 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 13b. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs receive:"
	log="$(${BTRFS} receive -f ./log1 ${MOUNTPOINT}/${MYSAVEDDIR} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed: ${log}"
	fi
	check_ioctl_cmds "${log}"

	# 14. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs balance start:"
	log="$(${BTRFS} balance start --full-balance ${MOUNTPOINT} 2>&1)"
	check_ioctl_cmds "${log}"

	# 15. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs balance status:"
	log="$(${BTRFS} balance status ${MOUNTPOINT} 2>&1)"
	check_ioctl_cmds "${log}"

	# 16. Case # 1
	tst_resm TINFO "Check ioctl commands on btrfs balance cancel:"
	log="$(${BTRFS} balance cancel ${MOUNTPOINT} 2>&1)"
	check_ioctl_cmds "${log}"

	if [[ "${image2_ok}" -eq 0 ]] ; then
		sleep 1
       		tst_exit
	fi
	# 1. Case # 2
	tst_resm TINFO "Check ioctl commands on btrfs device add:"
	log="$(${BTRFS} device add -f ${LOOP_DEV2} ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
		check_ioctl_cmds "${log}"
	else
		check_ioctl_cmds "${log}"
		# 2. Case # 2
		tst_resm TINFO "Check ioctl commands on btrfs device remove:"
		log="$(${BTRFS} device remove ${LOOP_DEV2} ${MOUNTPOINT} 2>&1)"
		if [[ $? -ne 0 ]] ; then
			tst_resm TINFO "${BTRFS} failed"
		fi
		check_ioctl_cmds "${log}"
	fi
	sleep 1

	# 3. Case # 2
	tst_resm TINFO "Check ioctl commands on btrfs replace start:"
	log="$(${BTRFS} replace start -f ${LOOP_DEV} ${LOOP_DEV2} ${MOUNTPOINT} 2>&1)"
	if [[ $? -ne 0 ]] ; then
		tst_resm TINFO "${BTRFS} failed"
	fi
	check_ioctl_cmds "${log}"
	sleep 3

	# 4. Case # 2
	tst_resm TINFO "Check ioctl commands on btrfs replace status:"
	log="$(${BTRFS} replace status -1 ${MOUNTPOINT} 2>&1)"
	check_ioctl_cmds "${log}"
	sleep 1

	# 5. Case # 2
	tst_resm TINFO "Check ioctl commands on btrfs replace cancel:"
	log="$(${BTRFS} replace cancel ${MOUNTPOINT} 2>&1)"
	check_ioctl_cmds "${log}"
	sleep 1

	tst_resm TINFO "SUMMARY:"
	tst_resm TINFO "  PASSED: ${TST_PASSED}"
	tst_resm TINFO "  FAILED: ${TST_FAILED}"

	tst_exit
}

setup
TST_CLEANUP=cleanup

do_test
