#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)
progdir=$(basename ${PWD})
CWD=${PWD}
PKGDIR=${CWD}/pkg

# This have to be set in command line
TARGET=""
SOURCES=""
#[ x"$CC" == x ] && CC=${CROSS_COMPILE}gcc

###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progdir}/${progname} === $*"
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname -t TARGET -s SOURCES"
	exit 1;
}

###############################################################################
# Make all
###############################################################################
makeAll()
{
	[ x"${CC}" != x ] && OPT_CC="CC=$CC" || OPT_CC="CC=gcc"
	[ x"$ARCH" != x ] && [ x"$archm" != x"$ARCH" ] && OPT_HOST="--host=$ARCH"
	doEcho "OPT_HOST = ${OPT_HOST}"
	doEcho "OPT_CC   = ${OPT_CC}"

	cwd="$(pwd)"
	cd ${SOURCES} 
	./configure "$OPT_HOST"
	make all "$OPT_CC" || exit 1
	cd ${cwd}

	cp ./conf/configure.sh ./conf/configure
	chmod a+x ./conf/configure
}


###############################################################################
# Make clean
###############################################################################
makeClean()
{
	make clean -C ${SOURCES}
}


###############################################################################
# Install to directory
###############################################################################
installToDir()
{
	INSTDIR=$1
	if [ ! -d "${INSTDIR}" ]; then
		doEcho "Directory <${INSTDIR}> does not exist"
		return 1
	fi
	cp -r ${SOURCES}/* ${INSTDIR}
}


###############################################################################
# Make install
###############################################################################
makeInstall()
{
	instdir="${CWD}/root"
	mkdir -p ${instdir}
	installToDir ${instdir}
}


###############################################################################
# Make package
###############################################################################
makePackage()
{
	exit 1
}


###############################################################################
# Make uninstall
###############################################################################
makeUninstall()
{
	rm -rf "${CWD}/root"
	rm -f ./conf/configure
}


###############################################################################
# Make clobber
###############################################################################
makeClobber()
{
	make distclean -C ${SOURCES}
	rm -f ./*~
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
for param in "$@"
do
	case "${param}" in
	CC=*)
		p=${param}; p=${p#CC=}; p=${p#\"}; p=${p%\"}; CC=$p
	;;
	*)
			
	;;
	esac
done

while getopts ":t:s:" optname
do
	case "$optname" in
	"t")
		TARGET=$OPTARG
	;;
	"s")
		SOURCES=$OPTARG
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# Check input parameters
###############################################################################
[ -z "${TARGET}" -o -z "${SOURCES}" ] && usage

if [ ! -d "${SOURCES}" ]; then
	doEcho "Directory <${SOURCES}> does not exist"
	exit 1
fi

archm=`uname -m | sed -e s/i.86/i386/ -e s/sun4u/sparc64/`

VERSION="$(head -n 1 ${SOURCES}/ChangeLog)"

###############################################################################
# Print settings
###############################################################################
doEcho "VERSION = ${VERSION}"
doEcho "ARCH = ${ARCH}"
doEcho "TARGET = ${TARGET}"
doEcho "CC = ${CC}"


###############################################################################
# Switch target
###############################################################################
case "${TARGET}" in
	"all"|"a")
		makeAll
	;;
	"install"|"i")
		makeInstall
	;;
	"package"|"pkg")
		makePackage
	;;
	"clean"|"c")
		makeClean
	;;
	"uninstall"|"u")
		makeUninstall
	;;
	"clobber"|"cl")
		makeClobber
	;;
	*)
		doEcho "Unknown target <${TARGET}>"	
	;;
esac
