#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <unistd.h>
#include <pthread.h>

#define PASS 0
#define FAIL 1
#define SKIP 2

struct demo_str_a {
	char c;
	int i;
	long l;
	char* str;
};

/* ######################################################################################
 * # NB> Structures 'a' above and 'b' below are pretty similar, but 'b' is much bigger. #
 * ######################################################################################
 */

struct demo_str_b {
	char c;
	int i;
	long l;
	char __pad[128];
	char* str;
};

int demo1(void)
{
	struct demo_str_a *psa = malloc(sizeof(struct demo_str_a));
	struct demo_str_b *psb = malloc(sizeof(struct demo_str_b));
	int wstatus = 0;
	int pid = 0;

	printf("%s: partially initialized structure leads to SIGILL\n", __func__);

	psa->c = 'a';		psb->c = 'b';
	psa->str = "sa";	psb->str = "sb";
	psa->l = 0;		psb->l = 10;
	
	pid = fork();
	if (pid < 0) {
		printf("%s: fork() failed\n", __func__);		
		return FAIL;
        } else if (pid == 0) {  /* child */
		if (psa->i == psb->l) // <-- typo: wrong field name ('i' instead of 'l')
			printf("\t--> it breaks at line %d !!!\n", __LINE__);
        	exit(PASS);               
	}

	wait(&wstatus);
        if (WIFSIGNALED(wstatus) && WTERMSIG(wstatus) == SIGILL) {
		printf("%s: Child killed by SIGILL as expected", __func__);
		return PASS;
	}

	free(psa);
	free(psb);
	
	return FAIL;
}


int demo2_bad_index = 0; /* to report out */
int demo2_expected_bad_index = 0; /* to report out */
/*
 * handler()
 *	A signal handler to catch expected signals.
 */
void handler_d2(int sig)
{
	char* excName;

	switch (sig) {
	case SIGSEGV:
		excName = "SIGSEGV";
		break;
	default:
		excName = "!!!UNEXPECTED!!!";
		break;
	}
	printf("failed\n");
	printf("\tIndexing array[%d] ended up with exception #%d (\"%s\").\n",
	       demo2_bad_index, sig, excName);
	fflush(stdout);

// NB> We let it to fail when return back; therefore no 'exit' below.
// 	exit(demo2_bad_index);
}
/*
 * set_handler()
 * 	Establish a signal handler for "sig" with the specified flags and
 * 	signal to mask while the handler executes.
 * Returns
 *	0 on success, errno on failure
 */
int set_handler(int sig, int sig_to_mask, void *handler)
{
	struct sigaction sa;
	int err;

	memset(&sa, 0, sizeof(sa));

	sa.sa_sigaction = handler;
	sa.sa_flags = SA_RESTART | SA_RESETHAND | SA_SIGINFO;
	sigemptyset(&sa.sa_mask);
	sigaddset(&sa.sa_mask, sig_to_mask);
	err = sigaction(sig, &sa, NULL);

	if (err == 0)
		return 0;
	else
		return errno;
}

#define DEBUG 0
void *ArrBoundTest(void *ptr, int index)
{
	char *str = (char *) ptr;
	char character;

	if (DEBUG) {
		printf("Entered test func \"%s\"...\n", __func__);
	}

	demo2_expected_bad_index = index >= 0 ? __builtin_e2k_get_ap_size(str) : -1;
	demo2_bad_index = index;
	printf("\t%s :: index=%d...", __func__, index);
	fflush(stdout);
	character = str[index];

	if (index < __builtin_e2k_get_ap_size(str)) { // <-- array bounds ?
		printf("passed...array[%d]='%c'\n", index, character);
		fflush(stdout);
	} else if (index < 0 || index >= __builtin_e2k_get_ap_size(str)) {
		printf("\t\tSurprisingly SIGSEGV exception wasn't catch (index=%d)...\n",
		       index);
	}

	return  ptr;
 }
int demo2(void *array, int index)
{
	pthread_t pthr;
	char *str = (char *) array;
	int wstatus = 0;
	int pid = 0;

	demo2_expected_bad_index = (index < 0) ? index : __builtin_e2k_get_ap_size(str);

	if (index <= 0) {
		printf("%s%s: attempt to access out of array bounds leads to SIGSEGV\n",
			__func__, ((index >= 0) ? "a" : "b"));
		printf("\t Capacity of the array is [%d]:\n", __builtin_e2k_get_ap_size(str));
		printf("\t\tSIGSEGV exception expected when index is '%d'...\n",
		       (index ? index : __builtin_e2k_get_ap_size(str)));
	}

	if (set_handler(SIGSEGV, 0, (void *)handler_d2))
		return FAIL;

	pid = fork();
	if (pid < 0) {
		printf("%s: fork() failed\n", __func__);
		return FAIL;
        } else if (pid == 0) {  /* child */
		char *ptr = ArrBoundTest(array, index);
        	exit(FAIL);
	}

	wait(&wstatus);
 	(void)set_handler(SIGSEGV, 0, (void *)SIG_DFL);
	if (WIFSIGNALED(wstatus) && WTERMSIG(wstatus) == SIGSEGV) {
		if (index != demo2_expected_bad_index) {
			printf("%s: Child killed by SIGSEGV at unexpected index '%d'\n",
			       __func__, demo2_bad_index);
			return FAIL;
		}
		printf("%s: Child killed by SIGSEGV at index '%d' as expected",
		       __func__, index);
		return PASS;
	} else 	if (!WIFSIGNALED(wstatus) && index < __builtin_e2k_get_ap_size(str)) {
		return PASS;
	} else {
		printf("!!! Indexing array[%d]: child exites with status 0x%x (\"%s\") !!!\n",
		       index, wstatus);
	}

	return FAIL;
}

int demo3a(void)
{
	struct demo_str_a sa;
	struct demo_str_b sb;
	struct demo_str_a *psa = &sa;
	struct demo_str_b *psb = &sb;
	int wstatus = 0;	
	int pid = 0;

	printf("%s: wrong type cast can lead to SIGSEGV\n", __func__);

	sa.c = 'a';	sb.c = 'b';
	sa.str = "sa";	sb.str = "sb";
	sa.l = 0;	sb.l = 10;

	pid = fork();
	if (pid < 0) {
		printf("%s: fork() failed\n", __func__);		
		return FAIL;
        } else if (pid == 0) {  /* child */
		psb = (struct demo_str_b *) psa; // <-- wrong pointer type used
		psb->str = psa->str;
		printf("\t--> In PM it breaks at line %d !!!\n", __LINE__);
		exit(PASS);               
	}

	wait(&wstatus);
        if (WIFSIGNALED(wstatus) && WTERMSIG(wstatus) == SIGSEGV) {
		printf("%s: Child killed by SIGSEGV as expected", __func__);
		return PASS;
	}
	
	return FAIL;
}

int demo3b(void)
{
	struct demo_str_a sa;
	struct demo_str_b sb;
	struct demo_str_a *psa = &sa;
	struct demo_str_b *psb = &sb;
	int wstatus = 0;	
	int pid = 0;

	printf("%s: wrong pointer manipulations lead to SIGSEGV\n", __func__);

	sa.c = 'a';	sb.c = 'b';
	sa.str = "sa";	sb.str = "sb";
	sa.l = 0;	sb.l = 10;

	pid = fork();
	if (pid < 0) {
		printf("%s: fork() failed\n", __func__);		
		return FAIL;
        } else if (pid == 0) {  /* child */
		psb = (struct demo_str_b *) ((char *)psa + 8); // <-- wrong pointer manipulation
		if (psb->str != psa->str)
			printf("\t--> In PM it breaks at line %d !!!\n", __LINE__);
		exit(PASS);               
	}

	wait(&wstatus);
        if (WIFSIGNALED(wstatus) && WTERMSIG(wstatus) == SIGSEGV) {
		printf("%s: Child killed by SIGSEGV as expected", __func__);
		return PASS;
	}
	
	return FAIL;
}

int demo4(void)
{
	char buf[128];
	char *rp1 /* = NULL */;
	char *rp2 = NULL;
	char *foo = "tst-sscanf test string";
	int ret;
	int wstatus = 0;	
	int pid = 0;

	printf("%s: sscanf never returns descriptor but pointer (address) instead\n", __func__);

	memset(&buf, 0, sizeof(buf));

	snprintf(buf, 128, "%p", foo);
	printf("tst-sscanf: buf = [%s]\n", buf);

	pid = fork();
	if (pid < 0) {
		printf("%s: fork() failed\n", __func__);		
		return FAIL;
        } else if (pid == 0) {  /* child */
		/* int sscanf(const char *str, const char *format, ...); */
		ret = sscanf(buf, "%p", &rp1);
		ret += sscanf(buf, "%p", &rp2);
		if (ret < 2 || (rp1[0] != '0') || (rp1[1] != 'x'))
			printf("In PM sscanf() failed over here\n");
		exit(PASS);               
	}

	wait(&wstatus);
        if (WIFSIGNALED(wstatus) && WTERMSIG(wstatus) == SIGILL) {
		printf("%s: Child killed by SIGILL as expected", __func__);
		return PASS;
	}
	
	return FAIL;

}

int
main (int argc, char **argv)
{
	int i;
#ifdef __ptr128__
	printf("((((( %s: Running in protected (128 bit pointer) execution mode ... )))))\n\n", __FILE__);
#endif

	if ( demo1() == PASS ) {
		printf("\n%s: demo1 : PASS\n", __func__);
	}
	else {
		printf("\n%s: demo1 : FAIL\n", __func__);
		return FAIL;
	}

	for (i = 0; i <=3; i++) {
		if (demo2("sa", i) != PASS ) {
			printf("\n%s: demo2a : FAILED @ index=%d\n", __func__, i);
			break;
			return FAIL;
		}
	}
	printf("\n%s: demo2a : PASS\n", __func__);

	if ( demo2("sa", -1) == PASS ) {
		printf("\n%s: demo2b : PASS\n", __func__);
	}
	else {
		printf("\n%s: demo2b : FAIL\n", __func__);
		return FAIL;
	}

	if ( demo3a() == PASS ) {
		printf("\n%s: demo3a : PASS\n", __func__);
	}
	else {
		printf("\n%s: demo3a : FAIL\n", __func__);
		return FAIL;
	}

	if ( demo3b() == PASS ) {
		printf("\n%s: demo3b : PASS\n", __func__);
	}
	else {
		printf("\n%s: demo3b : FAIL\n", __func__);
		return FAIL;
	}

	if ( demo4() == PASS ) {
		printf("\n%s: demo4 : PASS\n", __func__);
	}
	else {
		printf("\n%s: demo4 : FAIL\n", __func__);
		return FAIL;
	}

	return PASS;
}
