#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include "common.h"

/*******************************************************************************
********************************************************************************
*******************************************************************************/

#define	UP_TO_MIDDLE		0
#define UP_TO_TOP		1
#define UP_TO_BOTTOM		2
#define	MIN_MIDDLE_FUNC_LEVEL	1
#define	MAX_MIDDLE_FUNC_LEVEL	16
#define	MIN_TOP_LEVEL		1
#define	MAX_TOP_LEVEL		32
#define	TOP_LEVEL_INCR		4
#define	MIN_BOTTOM_LEVEL	1
#define	BOTTOM_LEVEL_DECR	5

typedef	unsigned long		u64;
typedef	unsigned int		u32;
typedef	unsigned short		u16;
typedef	unsigned char		u8;
/******************************************************************************/
static	char	*tc_name = "hard_stack_tc";
static	int	FAIL = 0;
static	int	VERBOSE = 0;
static	int	DESC = 0;
static	int	middle_level;
static	int	current_level;
static	int	top_level;
static	int	bottom_level;
static	pid_t	pid;
static	int	parent;
/******************************************************************************/
static void tp_001();
static void parse_cmdline(int argc, char **argv);

/*******************************************************************************
********************************************************************************
*******************************************************************************/

/*******************************************************************************
*   main
*******************************************************************************/
int
main(int argc, char **argv, char **envr)
{
	start_test(tc_name);
	parse_cmdline(argc, argv);

	tp_001();

	finish_test(FAIL);
	return FAIL;
}

/*******************************************************************************
*   parse_cmdline
*******************************************************************************/
void
parse_cmdline(int argc, char **argv)
{
	int i;
	for(i=0; i < argc; i++){
		if (strcmp(argv[i], "-v") == 0){
			DESC = 1;
			VERBOSE = 1;
			break;
		}
	}
}

/*******************************************************************************
*   nesting_func0
*******************************************************************************/
static int
nesting_func0(int to_level, int cur_level, int direction, int child_flag)
{
	int	ret;
	long	long_cur_level;
	short	short_cur_level;
	char	char_cur_level;
	u64	u64_cur_level;
	u32	u32_cur_level;
	u16	u16_cur_level;
	u8	u8_cur_level;

	current_level ++;
	cur_level ++;
	long_cur_level = cur_level;
	short_cur_level = cur_level;
	char_cur_level = cur_level;
	u64_cur_level = cur_level;
	u32_cur_level = cur_level;
	u16_cur_level = cur_level;
	u8_cur_level = cur_level;

	if (VERBOSE) {
		if (direction == UP_TO_MIDDLE) {
			log_printf("#MLTC# {%s}{nesting_func0}"
				   " Up to middle level %d current level %d\n",
				   tc_name, to_level, cur_level);
		} else if (direction == UP_TO_TOP) {
			log_printf("#MLTC# {%s}{nesting_func0}"
				   " Up to top level %d current level %d\n",
				   tc_name, to_level, cur_level);
		} else if (direction == UP_TO_BOTTOM) {
			log_printf("#MLTC# {%s}{nesting_func0}"
				   " Up to bottom level %d current level %d\n",
				   tc_name, to_level, cur_level);
		} else {
			log_printf("#MLTC# {%s}{nesting_func0}"
				   " Unknown direction to level %d"
				   " current level %d\n",
				   tc_name, to_level, cur_level);
		}
	}
	if (direction == UP_TO_MIDDLE) {
		if (cur_level != current_level ||
			(long)cur_level != long_cur_level ||
			(short)cur_level != short_cur_level ||
			(char)cur_level != char_cur_level ||
			(u64)cur_level != u64_cur_level ||
			(u32)cur_level != u32_cur_level ||
			(u16)cur_level != u16_cur_level ||
			(u8)cur_level != u8_cur_level) {
			FAIL = 1;
			log_printf("#MLTC# {%s}{nesting_func0}"
				   " nested function call"
				   " local and global current level should\n"
				   "be equal"
				   " local and global current level are"
				   " different\n",
				   tc_name);
			return (-1);
		}
		if (parent != -1) {
			if (parent == child_flag) {
				FAIL = 1;
				log_printf("#MLTC# {%s}{nesting_func0}"
				   " nested function call"
				   " parent and child flags should\n"
				   "be different"
				   " parent and child flags are"
				   " equal\n",
				   tc_name);
				return (-1);
			}
		}
		while (1) {
			if (current_level < middle_level) {
				ret = nesting_func0(to_level, cur_level,
						UP_TO_MIDDLE, child_flag);
				if (ret != 0)
					return (ret);
				current_level --;
				if (cur_level != current_level ||
					(long)cur_level != long_cur_level ||
					(short)cur_level != short_cur_level ||
					(char)cur_level != char_cur_level ||
					(u64)cur_level != u64_cur_level ||
					(u32)cur_level != u32_cur_level ||
					(u16)cur_level != u16_cur_level ||
					(u8)cur_level != u8_cur_level) {

					FAIL = 1;
					log_printf("#MLTC# {%s}{nesting_func0}"
				   		" nested function return"
				   		" local and global current\n"
				   		"level should be equal"
				   		" local and global current"
				   		" level are different\n",
				   		tc_name);

					return (-1);
				}
				if (parent != -1) {
					if (parent == child_flag) {
						FAIL = 1;
						log_printf("#MLTC# {%s}"
							   "{nesting_func0}"
				   			   " nested function"
				   			   " return"
				   			   " parent and child\n"
				   			   "flags should"
				   			   " be different\n"
							   " parent and child"
							   " flags are equal\n",
				   			   tc_name);
						return (-1);
					}
				}
				if (current_level > bottom_level) {
					if (VERBOSE)
						log_printf("#MLTC# {%s}"
							   "{nesting_func0}"
				   			   " Continue to return"
				   			   " down from middle"
				   			   " level %d current"
				   			   " level %d\n",
				   			   tc_name,
							   middle_level,
							   cur_level);
					return (0);
				}
				if (VERBOSE)
					log_printf("#MLTC# {%s}{nesting_func0}"
						   " Returned to bottom level %d"
				   		   " from middle level %d\n",
				   		   tc_name,
						   cur_level, middle_level);
				if (cur_level != bottom_level) {
					FAIL = 1;
					log_printf("#MLTC# {%s}{nesting_func0}"
						   " nested function call"
				   		   " local and global"
						   " bottom level"
						   " should be equal\n"
						   "local and global bottom level"
						   " are different\n",
				   		   tc_name);
					return (-1);
				}
				if (VERBOSE)
					log_printf("#MLTC# {%s}{nesting_func0}"
						   " Start to call functions to"
				   		   " middle level %d"
 						   " current level %d\n",
				   		   tc_name,
						   middle_level, cur_level);
				continue;
			}
			if (to_level != middle_level) {
				FAIL = 1;
				log_printf("#MLTC# {%s}{nesting_func0}"
					   " nested function call"
					   " local and global"
					   " middle level"
					   " should be equal\n"
					   "local and global middle level"
					   " are different\n",
					   tc_name);
				return (-1);
			}
			if (top_level >= middle_level + MAX_TOP_LEVEL) {
				if (VERBOSE)
					log_printf("#MLTC# {%s}{nesting_func0}"
						   " Max top level reached:"
				   		   " return to choose"
						   "  new middle level\n",
						   tc_name);
				bottom_level = 0;
				return (0);
			}
			if (top_level == 0) {
				top_level = middle_level + MIN_TOP_LEVEL;
				bottom_level = middle_level - MIN_BOTTOM_LEVEL;
				pid = fork();
				if (pid < 0) {
					FAIL = 1;
					log_printf("#MLTC# {%s}{nesting_func0}"
						   " System call fork() failed ;"
						   " errno = %d\n",
					   	   tc_name, errno);
					return (-1);
				} else if (pid == 0) {	/* child */
					parent = 0;
					child_flag = 1;
					if (VERBOSE)
						log_printf("#MLTC# {%s}"
							   "{nesting_func0}"
						   	   " fork() succeeded"
							   " and CHILD is here\n",
					   	   	   tc_name);
				} else {
					parent = 1;
					child_flag = 0;
					if (VERBOSE)
						log_printf("#MLTC# {%s}"
							   "{nesting_func0}"
						   	   " fork() succeeded"
							   " and PARENT is here\n",
					   	   	   tc_name);
				}
			} else {
				top_level += TOP_LEVEL_INCR;
				bottom_level -= BOTTOM_LEVEL_DECR;
			}
			if (bottom_level <= 0)
				bottom_level = 1;
			ret = nesting_func0(top_level, cur_level, UP_TO_TOP,
				child_flag);
			if (ret != 0)
				return (ret);
			current_level --;
			if (VERBOSE)
				log_printf("#MLTC# {%s}{nesting_func0}"
					   " Returned to the middle level %d"
					   " from top level %d\n",
					   tc_name, middle_level, top_level);
			if (cur_level != current_level ||
				(long)cur_level != long_cur_level ||
				(short)cur_level != short_cur_level ||
				(char)cur_level != char_cur_level ||
				(u64)cur_level != u64_cur_level ||
				(u32)cur_level != u32_cur_level ||
				(u16)cur_level != u16_cur_level ||
				(u8)cur_level != u8_cur_level) {
				FAIL = 1;
				log_printf("#MLTC# {%s}{nesting_func0}"
					   " nested function return"
					   " local and global current\n"
					   "level should be equal"
					   " local and global current"
					   " level are different\n",
					   tc_name);
				return (-1);
			}
			if (parent != -1) {
				if (parent == child_flag) {
					FAIL = 1;
					log_printf("#MLTC# {%s}{nesting_func0}"
						   " nested function return"
					   	   " parent and child flags"
					   	   " should\nbe different"
					   	   " parent and child flags are"
					   	   " equal\n",
					   	   tc_name);
					return (-1);
				}
			}
			if (cur_level <= bottom_level)
				continue;
			if (VERBOSE)
				log_printf("#MLTC# {%s}{nesting_func0}"
					   " Start return to the bottom level %d"
				   	   " from middle level %d\n",
 					   tc_name, bottom_level, cur_level );
			return (0);
		}
	} else if (direction == UP_TO_TOP) {
		if (cur_level != current_level ||
			(long)cur_level != long_cur_level ||
			(short)cur_level != short_cur_level ||
			(char)cur_level != char_cur_level ||
			(u64)cur_level != u64_cur_level ||
			(u32)cur_level != u32_cur_level ||
			(u16)cur_level != u16_cur_level ||
			(u8)cur_level != u8_cur_level) {
			FAIL = 1;
			log_printf("#MLTC# {%s}{nesting_func0}"
				   " nested function call"
				   " local and global current level should\n"
				   "be equal"
				   " local and global current level are"
				   " different\n",
				   tc_name);
			return (-1);
		}
		if (parent != -1) {
			if (parent == child_flag) {
				FAIL = 1;
				log_printf("#MLTC# {%s}{nesting_func0}"
				   	   " nested function call"
				   	   " parent and child flags should\n"
				   	   "be different"
				   	   " parent and child flags are"
				   	   " equal\n",
				   	   tc_name);
				return (-1);
			}
		}
		if (current_level < top_level) {
			ret = nesting_func0(to_level, cur_level, UP_TO_TOP,
					child_flag);
			if (ret != 0)
				return (0);
			current_level --;
			if (VERBOSE)
				log_printf("#MLTC# {%s}{nesting_func0}"
					   " Return to middle level %d:"
				   	   " current level %d\n",
 					   tc_name, middle_level, cur_level );

			if (cur_level != current_level ||
				(long)cur_level != long_cur_level ||
				(short)cur_level != short_cur_level ||
				(char)cur_level != char_cur_level ||
				(u64)cur_level != u64_cur_level ||
				(u32)cur_level != u32_cur_level ||
				(u16)cur_level != u16_cur_level ||
				(u8)cur_level != u8_cur_level) {
				FAIL = 1;
				log_printf("#MLTC# {%s}{nesting_func0}"
					   " nested function return"
				   	   " local and global current level should\n"
					   "be equal"
					   "local and global current level are"
					   "different\n",
 					   tc_name );
				return (-1);
			}
			if (parent != -1) {
				if (parent == child_flag) {
					FAIL = 1;
					log_printf("#MLTC# {%s}{nesting_func0}"
						   " nested function return"
					   	   " parent and child flags should\n"
						   "be different"
						   "parent and child flags are"
						   "equal\n",
 					   	   tc_name );
					return (-1);
				}
			}
			return (0);
		}
		if (to_level != top_level) {
			FAIL = 1;
			log_printf("#MLTC# {%s}{nesting_func0}"
				   " nested function call"
			   	   " local and global top level should\n"
				   "be equal"
				   "local and global middle level are"
				   "different\n",
			   	   tc_name );
			return (-1);
		}
		if (VERBOSE)
			log_printf("#MLTC# {%s}{nesting_func0}"
					   " Start return to middle level %d:"
				   	   " current level %d\n",
 					   tc_name, middle_level, cur_level );
		return (0);
	}
	return (0);
}

/*******************************************************************************
*   check_hard_stacks
*******************************************************************************/
static int
check_hard_stacks()
{
	int	ret;
	int	wait_ret;
	int	stat_loc;
	int	middle;

	if (VERBOSE)
		log_printf("#MLTC# {%s}{check_hard_stacks} test 'hardware stacks"
			   " increment & decrement': start\n", tc_name);

	for (middle = MIN_MIDDLE_FUNC_LEVEL; middle <= MAX_MIDDLE_FUNC_LEVEL;
								middle ++) {
		current_level = 0;
		middle_level = middle;
		top_level = 0;
		bottom_level = 0;
		
		if (VERBOSE)
			log_printf("#MLTC# {%s}{check_hard_stacks} Start nested"
				   " function call: middle level %d\n", 
							tc_name, middle_level);
		parent = -1;
		ret = nesting_func0(middle, 0, UP_TO_MIDDLE, -1);
		if ((parent == 1) && (pid != 0)) {
			if (VERBOSE)
				log_printf("#MLTC# {%s}{check_hard_stacks}"
			   		   " PARENT: start waiting for child\n",
					   tc_name);
			wait_ret = waitpid(pid, &stat_loc, 0);
			if (wait_ret != pid && errno != ECHILD) {
				FAIL = 1;
				log_printf("#MLTC# {%s}{check_hard_stacks}"
			   		   " waitpid(): wait_ret != pid\n",
					   tc_name);
				log_printf("#MLTC# {%s}{check_hard_stacks}"
			   		   " waitpid(): pid %d"
					   " wait_ret %d stat_loc %d\n",
					   tc_name, pid, wait_ret, stat_loc);
				return (-1);
			}
			if (VERBOSE){
				log_printf("#MLTC# {%s}{check_hard_stacks}"
					   " PARENT: child terminated\n",
					   tc_name);
				log_printf("#MLTC# {%s}{check_hard_stacks}"
					   " wait_ret %d stat_loc %d\n",
					   tc_name, wait_ret, stat_loc);
			}
			if (stat_loc != 0) {
				FAIL = 1;
				log_printf("#MLTC# {%s}{check_hard_stacks}"
					   " waitpid(): returned non zero"
					   " exit status\n", tc_name);
				return (-1);
			}
		} else if (parent == 0) {
			if (VERBOSE)
				log_printf("#MLTC# {%s}{check_hard_stacks}"
					   " CHILD: is exiting and returning %d",
					   tc_name, ret);
			_exit (ret);
		}
		if (ret != 0)
			return (ret);
	}
	return (0);
}

/*******************************************************************************
*   tp_001
*******************************************************************************/
void
tp_001()
{
	if (DESC)	
		log_printf("#MLTC# {%s}{tp_001}\n"
			"Hardware procedure stack and procedure chain stack\n"
			"allocation, expansion and constriction\n"
			"In a loop recursive call a function on a various levels\n"
			"from the minimal top level to the maximal nested level\n",
									tc_name);

	check_hard_stacks();
}
