#ifndef __MLTC_COMMON_H__
#define __MLTC_COMMON_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/mount.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdarg.h>
#include <errno.h>

/*******************************************************************************
********************************************************************************
*******************************************************************************/

#define	STRBUF_SIZE 1024
#define MLTC_FILE_NAME	"/usr/mltc_run_all_tc"

#ifdef __e2k__
	#include <asm/e2k_api.h>
	#include <asm/e2k_syswork.h>

	#define E2K_GET_TICK E2K_GET_DSREG(clkr)
#else
	#define E2K_GET_TICK get_nsec_time()
#endif
/******************************************************************************/
static char		cur_tc_name[STRBUF_SIZE];
static struct timeval	time_begin;
static struct timeval	time_finish;
static long		mksek;
/******************************************************************************/
void log_printf(char *format, ...);
/*******************************************************************************
********************************************************************************
*******************************************************************************/

/*******************************************************************************
*   start_test
*******************************************************************************/
void
start_test(const char *testcase_name)
{
	strcpy(cur_tc_name, testcase_name);
	gettimeofday(&time_begin, NULL);
	log_printf("#MLTC# START_TC: {%s} pid %d\n", cur_tc_name, getpid());
}

/*******************************************************************************
*   finish_test
*******************************************************************************/
void
finish_test(int res)
{
	gettimeofday(&time_finish, NULL);
	mksek = (time_finish.tv_sec * 1000000 + time_finish.tv_usec) - 
		   (time_begin.tv_sec * 1000000 + time_begin.tv_usec);
	
	if( !res )
		log_printf("#MLTC# {%s} STATUS = PASS\n", cur_tc_name);
	else
		log_printf("#MLTC# {%s} STATUS = FAIL\n", cur_tc_name);

	log_printf("#MLTC# FINISH_TC: {%s} time %ld sec %ld mks\n", 
							cur_tc_name,
							mksek / 1000000,
							mksek % 1000000);
	exit(res);
}

/*******************************************************************************
*   log_printf
*******************************************************************************/
void
log_printf(char *format, ...)
{
	va_list	ap;
	char	str_to_print[STRBUF_SIZE];

	va_start(ap, format);
	vsprintf(str_to_print, format, ap);
	
	printf(str_to_print);
	//perror(str_to_print);
	fflush(stdout);
}

/*******************************************************************************
*   mount_tst
*******************************************************************************/
int
mount_tst(void)
{
	int 		fd;
	int 		rval = 0;
	
	fd = open(MLTC_FILE_NAME, O_RDONLY);
	if (fd == -1) {
		rval = mount(NULL, "/", NULL, MS_MGC_VAL | MS_REMOUNT, NULL);
	}
	return (rval);
}

/*******************************************************************************
*   get_nsec_time
*******************************************************************************/
static inline 
long long 
get_nsec_time()
{
	long long nsec_time;
	static struct timeval   time;
	gettimeofday(&time, 0);
	nsec_time = (time.tv_sec * 1000000LL + time.tv_usec) * 1000;
	return(nsec_time);
}

#ifndef __e2k__
	static inline int pthread_get_cpu_id(void)
	{
		return (0);
	}
	static inline int pthread_get_cpus_num(void)
	{
		return (1);
	}
	static inline int pempty(void)
	{
		return (getpid());
	}
	static inline int pthread_cpu_bind(unsigned long int pthread_id,
				long cpu)
	{
		log_printf( "{pthread_cpu_bind} pthread_id %ld cpu %ld",
							pthread_id, cpu );
		return (0);
	}
	static inline int pthread_wakeup(pthread_cond_t *cond, pthread_t pthr)
	{
		log_printf( "{pthread_wakeup} cond %p pthr %d",
							cond, pthr );
		return (0);
	}
	static inline int pthread_switch(pthread_cond_t *cond, pthread_t pthr)
	{
		log_printf( "{pthread_switch} cond %p pthr %d",
							cond, pthr );
		return (0);
	}
	static inline int pthread_wait(pthread_cond_t *cond,
						pthread_mutex_t *mutex)
	{
		log_printf( "{pthread_switch} cond %p mutex %p",
							cond, mutex );
		return (0);
	}
#endif /*__e2k__*/

#endif /*__MLTC_COMMON_H__*/
