/* SPDX-License-Identifier: GPL-2.0 */
/* This is a test to check which system calls are available in the system
 * print_available_syscalls.c, v 1.1 04/19/20023.
 *
 * Copyright (C) 2023 MCST
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <sys/syscall.h>
#include <asm/protected_mode.h>
#include <signal.h>

/* System call names: */
const char *sys_call_ID_to_name[NR_syscalls] = {
	"restart_syscall", /* 0 */
	"exit",
	"fork",
	"read",
	"write",
	"open",		/* 5 */
	"close",
	"waitpid",
	"creat",
	"link",
	"unlink",	/* 10 */
	"execve",
	"chdir",
	"time",
	"mknod",
	"chmod",	/* 15 */
	"lchown",
	"old_break",	/* place holder / ni_syscall */
	"old_stat",	/* ditto */
	"lseek",
	"getpid",	/* 20 */
	"mount",
	"oldumount",
	"setuid",
	"getuid",
	"stime",	/* 25 */
	"ptrace",
	"alarm",
	"old_fstat",	/* place holder / ni_syscal */
	"pause",
	"utime",	/* 30 */
	"old_stty",	/* place holder / ni_syscal */
	"old_gtty",	/* ditto */
	"access",
	"nice",
	"old_ftime",	/* 35, place holder / ni_syscal */
	"sync",
	"kill",
	"rename",
	"mkdir",
	"rmdir",	/* 40 */
	"dup",
	"pipe",
	"times",
	"old_prof",	/* place holder / ni_syscal */
	"brk",		/* 45 */
	"setgid",
	"getgid",
	"signal",	/* place holder / ni_syscal / to be emulated by rt_sigaction() */
	"geteuid",
	"getegid",	/* 50 */
	"acct",
	"umount",	/* recycled never used phys() */
	"old_lock",	/* old lock syscall holder */
	"ioctl",

	"fcntl",	/* 55 */

	"old_mpx",	/* place holder / ni_syscal */
	"setpgid",
	"old_ulimit",	/* place holder / ni_syscal */
	"old_olduname",	/* ditto */
	"umask",	/* 60 */
	"chroot",
	"ustat",
	"dup2",
	"getppid",
	"getpgrp",	/* 65 */
	"setsid",
	"sigaction",	/* no sys_sigaction() */
	"sgetmask",
	"ssetmask",
	"setreuid",	/* 70 */
	"setregid",
	"sigsuspend",	/* place holder / ni_syscal */
	"sigpending",
	"sethostname",
	"setrlimit",	/* 75 */
	"old_getrlimit",
	"getrusage",
	"gettimeofday",
	"settimeofday",
	"getgroups",	/* 80 */
	"setgroups",
	"old_select",
	"symlink",
	"old_lstat",	/* place holder / ni_syscal */
	"readlink",	/* 85 */
	"olduselib", /* obsolete syscall */
	"swapon",
	"reboot",
	"old_readdir",
	"mmap",		/* 90 */
	"munmap",

	"truncate",
	"ftruncate",

	"fchmod",
	"fchown",	/* 95 */
	"getpriority",
	"setpriority",
	"old_profil",	/* place holder / ni_syscal */
	"statfs",
	"fstatfs",	/* 100 */
	"ioperm",
	"socketcall",
	"syslog",
	"setitimer",
	"getitimer",	/* 105 */

	"newstat",	/* in libc used in ptr64 mode */
	"newlstat",	/* in libc used in ptr64 mode */
	"newfstat",	/* in libc used in ptr64 mode */

	"uname",
	"iopl",		/* 110 */
	"vhangup",
	"old_idle",	/* place holder / ni_syscal */
	"vm86old",	/* ditto */
	"wait4",
	"swapoff",	/* 115 */
	"sysinfo",
	"ipc",
	"fsync",
	"sigreturn",	/* place holder / ni_syscal */
	"clone",	/* 120 */
	"setdomainname",
	"newuname",
	"modify_ldt",	/* place holder / ni_syscal */
	"adjtimex",
	"mprotect",	/* 125 */
	"sigprocmask",
	"create_module",/* place holder / ni_syscal */
	"init_module",
	"delete_module",
	"get_kernel_syms", /* 130 / place holder / ni_syscal */
	"quotactl",
	"getpgid",
	"fchdir",
	"bdflush",
	"sysfs",	/* 135 - obsolete */
	"personality",
	"old_afs",	/* place holder / ni_syscal */
	"setfsuid",
	"setfsgid",
	"llseek",	/* 140 */
	"getdents",
	"select",
	"flock",
	"msync",
	"readv",	/* 145 */
	"writev",
	"getsid",
	"fdatasync",
	"_sysctl",
	"mlock",	/* 150 */
	"munlock",
	"mlockall",
	"munlockall",
	"sched_setparam",
	"sched_getparam",   /* 155 */
	"sched_setscheduler",
	"sched_getscheduler",
	"sched_yield",
	"sched_get_priority_max",
	"sched_get_priority_min",  /* 160 */
	"sched_rr_get_interval",
	"nanosleep",
	"mremap",
	"setresuid",
	"getresuid",	/* 165 */
	"vm86",		/* place holder / ni_syscal */
	"query_module",	/* ditto */
	"poll",
	"nfsservctl",	/* 169 place holder / ni_syscal */
	"setresgid",	/* 170 */
	"getresgid",
	"prctl",
	"rt_sigreturn",	/* 173 */
	"rt_sigaction",
	"rt_sigprocmask",	/* 175 */
	"rt_sigpending",
	"rt_sigtimedwait",
	"rt_sigqueueinfo",
	"rt_sigsuspend",
	"pread64",		/* 180 */
	"pwrite64",
	"chown",
	"getcwd",
	"capget",
	"capset",	/* 185 */
	"sigaltstack",
	"sendfile64",
	"getpmsg",	/* 188 place holder / ni_syscal */
	"putpmsg",	/* 189 place holder / ni_syscal */
	"vfork",	/* 190 */
	"getrlimit",
	"mmap2",

	/* Entries 193-194 are for BITS_PER_LONG == 32; and this is 64 bit OS */
	"truncate64", /* 193 place holder / ni_syscal */
	"ftruncate64",	/* 194 ditto	*/

	"stat64",	/* 195 */
	"lstat64",
	"fstat64",

	"lchown",
	"getuid32",
	"getgid32",	/* 200 */
	"geteuid32",
	"getegid32",
	"setreuid32",
	"setregid32",
	"pidfd_send_signal",	/* 205 */
	"pidfd_open",
	"fchown",
	"setresuid32",
	"getresuid32",
	"setresgid32",	/* 210 */
	"getresgid32",
	"chown",
	"setuid",
	"setgid",
	"setfsuid",	/* 215 */
	"setfsgid",
	"pivot_root",
	"mincore",
	"madvise",
	"getdents64",	/* 220 */
	"fcntl64",	/* 221 */
	"core",		/* place holder / ni_syscal */
	"macctl",	/* 223 */
	"newfstatat",
	"emergency",	/* 225 place holder / ni_syscal */
	"e2k_setjmp",	/* ditto */
	"e2k_longjmp",	/* ditto */
	"e2k_syswork",
	"clone_thread",
	"e2k_longjmp2", /* 230 */
	"soft_debug",	/* place holder / ni_syscal */
	"setxattr",
	"lsetxattr",
	"fsetxattr",
	"getxattr",	/* 235 */
	"lgetxattr",
	"fgetxattr",
	"listxattr",
	"llistxattr",
	"flistxattr",	/* 240 */
	"removexattr",
	"lremovexattr",
	"fremovexattr",
	"gettid",
	"readahead",	/* 245 */
	"tkill",
	"sendfile64",
	"futex",
	"sched_setaffinity",
	"sched_getaffinity",	/* 250 */
	"pipe2",
	"set_backtrace",
	"get_backtrace",
	"access_hw_stacks",
	"el_posix",	/* 255 */
	"io_uring_setup",
	"io_uring_enter",
	"io_uring_register",
	"set_tid_address",
	"el_binary", /* 260 */
	"timer_create",
	"timer_settime",
	"timer_gettime",
	"timer_getoverrun",
	"timer_delete",	/* 265 */
	"clock_settime",
	"clock_gettime",
	"clock_getres",
	"clock_nanosleep",
	"msgget",	/* 270 */
	"msgctl",
	"msgrcv",
	"msgsnd",
	"semget",
	"semctl",	/* 275 */
	"semtimedop",
	"semop",
	"shmget",
	"shmctl",
	"shmat",	/* 280 */
	"shmdt",
	"open_tree",
	"move_mount",
	"rseq",
	"io_pgetevents", /* 285 */
	"accept4",
	"sched_setattr",
	"sched_getattr",
	"ioprio_set",	/* 289 */
	"ioprio_get",	/* 290 */
	"inotify_init",	/* 291 */
	"inotify_add_watch",
	"inotify_rm_watch",
	"io_setup",	/* 294 */
	"io_destroy",
	"io_getevents",
	"io_submit",
	"io_cancel",
	"fadvise64",
	"exit_group",	/* 300 */
	"lookup_dcookie",
	"epoll_create",
	"epoll_ctl",
	"epoll_wait",
	"remap_file_pages",
	"statfs64",
	"fstatfs64",
	"tgkill",
	"utimes",
	"fadvise64_64",	/* 310 */

	"vserver",	/*  place holder / ni_syscal
			 *	isn't implemented in the Linux 2.6.14 kernel
			 */
	"mbind",
	"get_mempolicy",
	"set_mempolicy",
	"mq_open",
	"mq_unlink",
	"mq_timedsend",
	"mq_timedreceive",
	"mq_notify",
	"mq_getsetattr", /* 320 */
	"kexec_load",
	"waitid",
	"add_key",
	"request_key",
	"keyctl",
	"mcst_rt",	/* place holder / ni_syscal */
	"getcpu",
	"move_pages",
	"splice",
	"vmsplice",	/* 330 */
	"tee",
	"migrate_pages",
	"utimensat",
	"rt_tgsigqueueinfo",
	"openat",
	"mkdirat",
	"mknodat",
	"fchownat",
	"unlinkat",
	"renameat",	/* 340 */
	"linkat",
	"symlinkat",
	"readlinkat",
	"fchmodat",
	"faccessat",
	"epoll_pwait",
	"signalfd4",
	"eventfd2",
	"recvmmsg",
	"RESERVED",	/* 350 - ni_syscal */
	"timerfd_create",
	"timerfd_settime",
	"timerfd_gettime",
	"preadv",
	"pwritev",
	"fallocate",
	"sync_file_range",
	"dup3",
	"inotify_init1",
	"epoll_create1",	/* 360 */
	"fstatat64",
	"futimesat",
	"perf_event_open",
	"unshare",
	"get_robust_list",	/* 365 */
	"set_robust_list",
	"pselect6",
	"ppoll",
	"setcontext",
	"makecontext",		/* 370 */
	"swapcontext",
	"freecontext",
	"fanotify_init",
	"fanotify_mark",
	"prlimit64",
	"clock_adjtime",
	"syncfs",
	"sendmmsg",
	"setns",
	"process_vm_readv",	/* 380 */
	"process_vm_writev",
	"kcmp",
	"finit_module",
	/* added in linux-4.4 */
	"renameat2",
	"getrandom",		/* 385 */
	"memfd_create",
	"bpf",
	"execveat",
	"userfaultfd",
	"membarrier",		/* 390 */
	"mlock2",
	/* added in linux-4.9 */
	"seccomp",
	"shutdown",
	"copy_file_range",
	"preadv2",		/* 395 */
	"pwritev2",

	/* free (unused) items */
	"RESERVED",		/* 397 - ni_syscall */
	"RESERVED",		/* 398 - ni_syscall */
	"RESERVED",		/* 399 - ni_syscall */

	"name_to_handle_at",	/* 400 */
	"open_by_handle_at",	/* 401 */
	"statx",		/* 402 */
	/* added for compatibility with x86_64 */
	"socket",	/* 403 */
	"connect",	/* 404 */
	"accept",	/* 405 */
	"sendto",	/* 406 */
	"recvfrom",	/* 407 */
	"sendmsg",	/* 408 */
	"recvmsg",	/* 409 */
	"bind",		/* 410 */
	"listen",	/* 411 */
	"getsockname",	/* 412 */
	"getpeername",	/* 413 */
	"socketpair",	/* 414 */
	"setsockopt",	/* 415 */
	"getsockopt",	/* 416 */

	/* free (unused) items */
	"RESERVED",	/* 417 - ni_syscall */
	"RESERVED",	/* 418 - ni_syscall */

	/* protected specific system calls entries */
	"arch_prctl",	/* 419 */
	"newuselib",	/* 420 __NR_newuselib */
	"rt_sigaction_ex", /* 421 */
	"get_mem",	/* 422 */
	"free_mem",	/* 423 */
	"clean_descriptors", /* 424 */
	"unuselib", /* 425 */

	"clone3",
	"fsopen",
	"fsconfig",
	"fsmount",
	"fspick",	/* 430 */
	"close_range",
	"openat2",
	"pidfd_getfd",
	"faccessat2",
	"process_madvise", /* 435 */
};

#define MSG_USAGE		0
#define MSG_WRONG_START		1
#define MSG_WRONG_END		2
#define MSG_WRONG_END_ABORTED	3
#define MSG_UNEXP_SC_ABORTED	4
#define MSG_LIST_M128		5
#define MSG_LIST_M64		6
#define MSG_TOTAL_NUM		7

#define MSG_SC_SKIPPED_TRMNTS	8
#define MSG_SC_SKIPPED_HANGS	9
#define MSG_E2K_SPECIFIC	10
#define MSG_FAILED2KILL_PID	11
#define MSG_SC_UNAVAILABLE	12
#define MSG_SC_AVAILABLE	13

#define MSGS_NUM		14

static char * messages_ENG[MSGS_NUM] = {
[MSG_USAGE]             = "Usage: %s {<start#> {<end#>}}\n",
[MSG_WRONG_START]       = "%s: wrong start syscall number \"%s\". Aborted.\n",
[MSG_WRONG_END]         = "%s: wrong end syscall number \"%s\". Limited to %d.\n",
[MSG_WRONG_END_ABORTED] = "%s: wrong end syscall number \"%s\". Aborted.\n",
[MSG_UNEXP_SC_ABORTED]  = "%s: unexpected arg \"%s\". Aborted.\n",
[MSG_LIST_M128]         = "===== List of syscalls supported in the protected execution mode: =====\n",
[MSG_LIST_M64]          = "===== List of syscalls supported in the regular execution mode: =====\n",
[MSG_TOTAL_NUM]         = "===== Total number of supported syscalls : %d =====\n\n",

[MSG_SC_SKIPPED_TRMNTS] = "--- Syscall #%ld/%s was skipped as it terminates the program ---\n",
[MSG_SC_SKIPPED_HANGS]  = "--- Syscall #%ld/%s was skipped as it hangs up the program ---\n",
[MSG_E2K_SPECIFIC]      = "+++ Special e2k-specific syscall #%ld/%s (skipped) +++\n",
[MSG_FAILED2KILL_PID]   = "\n!!!!! Failed to kill child (pid# %d) !!!!!\n\n",
[MSG_SC_UNAVAILABLE]    = ">>> Syscall #%ld/%s is not available on this platform <<<\n",
[MSG_SC_AVAILABLE]      = "Syscall #%ld/%s is available on this platform\n",
};

static char * messages_RU[MSGS_NUM] = {
[MSG_USAGE]             = "Usage: %s {<start#> {<end#>}}\n",
[MSG_WRONG_START]       = "%s: неправильно задан начальный номер \"%s\". Исполнение прервано.\n",
[MSG_WRONG_END]         = "%s: неправильно задан финальный номер \"%s\". Ограничено до %d.\n",
[MSG_WRONG_END_ABORTED] = "%s: неправильно задан финальныйномер \"%s\". Исполнение прервано.\n",
[MSG_UNEXP_SC_ABORTED]  = "%s: неожиданный аргумент \"%s\". Исполнение прервано.\n",
[MSG_LIST_M128]         = "===== СПИСОК СИСТЕМНЫХ ВЫЗОВОВ, ПОДДЕРЖИВАЕМЫХ В РБВ: =====\n",
[MSG_LIST_M64]          = "===== СПИСОК СИСТЕМНЫХ ВЫЗОВОВ, ПОДДЕРЖИВАЕМЫХ В ОБЫЧНОМ РЕЖИМЕ: =====\n",
[MSG_TOTAL_NUM]         = "===== ОБЩЕЕ КОЛИЧЕСТВО ПОДДЕРЖАННЫХ СИСТЕМНЫХ ВЫЗОВОВ: %d =====\n\n",

[MSG_SC_SKIPPED_TRMNTS] = "--- Системный вызов #%ld/%s не запускался, т.к. он прерывает исполнение программы ---\n",
[MSG_SC_SKIPPED_HANGS]  = "--- Системный вызов #%ld/%s не запускался, т.к. он вызывает зависание программы ---\n",
[MSG_E2K_SPECIFIC]      = "+++ Специальный служебный системный вызов платформы e2k #%ld/%s (не запускался) +++\n",
[MSG_FAILED2KILL_PID]   = "\n!!!!! Failed to kill child (pid# %d) !!!!!\n\n",
[MSG_SC_UNAVAILABLE]    = ">>> Системный вызов #%ld/%s отсутствует (не реализован для данной платформы) <<<\n",
[MSG_SC_AVAILABLE]      = "Системный вызов #%ld/%s реализован для данной платформы (отработал)\n",
};

static char **messages = messages_ENG;


/*
 * Setup PM error messaging language.
 * 'max_len' - maximum expected env var length.
 * Returns: -1 - if env.var. is not set;
 *           1 - this is RUssian lang;
 *           0 - this is ENGlish lang.
 */
static
unsigned long check_lang_setup(const char *env_var_name)
{
	char *env_val;

	env_val = getenv(env_var_name);
	if (!env_val)
		return -1;

	/* This is check for RUSSIAN language setup:
	 * ru_RU.KOI8-R, ru_RU.KOI8_R, ru_RU.KOI8R
	 * ru_RU.UTF-8, ru_RU.UTF_8, ru_RU.UTF8, ru_RU
	 */
	if (strstr(env_val, "RU"))
		return 1;
	else
		return 0;

	return 0;
}


static int print_available_SC_nums(int start, int finish)
// Returns total number of available syscalls
{
	int tnum = 0;
	long ret;
	long i;

	for (i = start; i <= finish; i++) {
		if (	(i == __NR_exit_group) ||
			(i == __NR_sigreturn) ||
			(i == __NR_exit) ||
			(i == __NR_vhangup) ||
			(i == __NR_shutdown)
		   ) {
			printf(messages[MSG_SC_SKIPPED_TRMNTS], i, sys_call_ID_to_name[i]);
			tnum++;
			continue;
		}
		else if (i == __NR_pause) {
			printf(messages[MSG_SC_SKIPPED_HANGS], i, sys_call_ID_to_name[i]);
			tnum++;
			continue;
		}
		else if ((i == __NR_el_binary) ||
			 (i == __NR_e2k_syswork)
			) {
			printf(messages[MSG_E2K_SPECIFIC], i, sys_call_ID_to_name[i]);
// 			tnum++;
			continue;
		}
		else if ((i == __NR_fork) || (i == __NR_vfork)) {
			pid_t pid;

			pid = syscall(i);
			if (pid == 0) {
				_exit(0);
				return 0;
			}

			if (pid) {
				if (kill(pid, SIGKILL) == -1)
					printf(messages[MSG_FAILED2KILL_PID], pid);
			}
			ret = pid;
		}
		else if (i == __NR_signal) {
			ret = syscall(__NR_signal, SIGUSR1, SIG_DFL);
		}
		else if ((i == __NR_process_madvise) ||
			(i == __NR_bpf) ||
			(i == __NR_kexec_load) ||
			(i == __NR_utime) ||
			(i == __NR_accept4) ||
			(i == __NR_clean_descriptors)
			) {
			ret = syscall(i,  0,  0,  0,  0,  0,  0);
		}
		else {
			ret = syscall(i, -1, -1, -1, -1, -1, -1);
		}

		if (ret == -1 && errno == ENOSYS) {
			printf(messages[MSG_SC_UNAVAILABLE], i, sys_call_ID_to_name[i]);
		} else {
			printf(messages[MSG_SC_AVAILABLE], i, sys_call_ID_to_name[i]);
			tnum++;
		}
	}

	return tnum;
}


int main(int argc, char **argv)
{
	int total_num;
	int start = 1, finish = NR_syscalls - 1;
	int lang;

	if (argc > 1) { /* there is start sysnum arg */
		if (!strcmp(argv[1], "h") || !strcmp(argv[1], "-h")) {
			printf(messages[MSG_USAGE], argv[0]);
			exit(0);
		}
		start = atoi(argv[1]);
		if (start <= 0 || start >= NR_syscalls) {
			printf(messages[MSG_WRONG_START], argv[0], argv[1]);
			exit(-1);
		}
	}

	if (argc > 2) { /* there is end sysnum arg */
		finish = atoi(argv[2]);
		if (finish <= 0 || finish >= NR_syscalls || finish < start) {
			if (finish >= NR_syscalls && finish > start) {
				finish = NR_syscalls - 1;
				printf(messages[MSG_WRONG_END], argv[0], argv[2], finish);
			} else {
				printf(messages[MSG_WRONG_END_ABORTED], argv[0], argv[2]);
				exit(-2);
			}
		}
	}

	if (argc > 3) { /* unexpected arg */
		printf(messages[MSG_UNEXP_SC_ABORTED], argv[0], argv[3]);
		exit(-3);
	}

	/* Setting up proper message language: */
	lang = check_lang_setup("LC_ALL");
	if (lang < 0) {
		lang = check_lang_setup("LC_MESSAGES");
		if (lang < 0)
			lang = check_lang_setup("LANG");
	}
	if (lang >= 0)
		messages = lang ? messages_RU : messages_ENG;

#ifdef __ptr128__
	printf(messages[MSG_LIST_M128]);
	syscall(__NR_arch_prctl, PR_PM_DBG_MODE_SET,
				PM_SC_DBG_MODE_DEFAULT |
				PROTECTED_MODE_SOFT |
				PM_SC_DBG_MODE_NO_ERR_MESSAGES);
#else
	printf(messages[MSG_LIST_M64]);
#endif
	total_num = print_available_SC_nums(start, finish);

	printf(messages[MSG_TOTAL_NUM], total_num);

	exit(total_num);
}
