#include <linux/watchdog.h>
#include <unistd.h>
#include "common.h"

/*******************************************************************************
********************************************************************************
*******************************************************************************/

/*******************************************************************************
 * FIXME : this defines have to be imported from kernel include files
 ******************************************************************************/
#define MISC_MAJOR		10
#define WATCHDOG_MINOR		130

/******************************************************************************/
static	char	*tc_name = "watchdog_tc";
static	int		FAIL = 0;
static	int		VERBOSE = 0;

static char		*wd_dev_name = "/dev/watchdog";
static int		wd_fd = 0;
/******************************************************************************/
static void setup();
static void tp_001();
static void tp_002();
static void tp_003();
static void tp_004();
static void tp_005();
static void parse_cmdline(int argc, char **argv);

/*******************************************************************************
********************************************************************************
*******************************************************************************/

/*******************************************************************************
*   main
*******************************************************************************/
int
main(int argc, char **argv, char **envr)
{
	start_test(tc_name);
	parse_cmdline(argc, argv);

	setup();
	tp_001();
	tp_002();
	tp_003();
	tp_004();
	tp_005();

	finish_test(FAIL);
	return FAIL;
}

/*******************************************************************************
*   parse_cmdline
*******************************************************************************/
void
parse_cmdline(int argc, char **argv)
{
	int i;
	for(i=0; i < argc; i++){
		if (strcmp(argv[i], "-v") == 0){
			VERBOSE = 1;
			break;
		}
	}
}

/******************************************************************************
* setup
******************************************************************************/
void 
setup()
{
	char cmd[128];

	/* Remove watchdog device file, if exist */
	sprintf(cmd, "rm -f %s", wd_dev_name);
	if (system(cmd) < 0){
		log_printf("#MLTC# {%s}{setup} Cannot remove "
				   "watchdog device file\n", tc_name);
	}

	/* Create watchdog device file */
	sprintf(cmd, "mknod %s c %d %d", wd_dev_name, MISC_MAJOR, WATCHDOG_MINOR);
	if (system(cmd) < 0){
		log_printf("#MLTC# {%s}{setup} Cannot create "
		           "watchdog device file\n", tc_name);
		FAIL = 1;
		return;
	}
}

/*******************************************************************************
*   tp_001
*******************************************************************************/
void
tp_001()
{
	int timeout;

	log_printf("#MLTC# {%s}{tp_001} Test verifies watchdog "
	           "\"magic close\" function\n",
			 	tc_name);

	/* Open watchdog device file */
	wd_fd = open(wd_dev_name, O_WRONLY);
	if (wd_fd < 0){
		log_printf("#MLTC# {%s}{tp_001} Cannot open watchdog device\n", tc_name);
		FAIL = 1;
		return;
	}

	timeout = 2;
	if (ioctl(wd_fd, WDIOC_SETTIMEOUT, &timeout) < 0){
		log_printf("#MLTC# {%s}{tp_001} ioctl "
				   "WDIOC_SETTIMEOUT failed\n", tc_name);
		FAIL = 1;
		return;
	}
	
	if (write(wd_fd, "V", 1) != 1){
		log_printf("#MLTC# {%s}{tp_001} write magic failed\n", tc_name);
		FAIL = 1;
		return;
	}

	if (close(wd_fd) <0){
		log_printf("#MLTC# {%s}{tp_001} close failed\n", tc_name);
		FAIL = 1;
		return;
	}

	sleep(timeout + 1);

	log_printf("#MLTC# {%s}{tp_001} PASS\n", tc_name);
}

/*******************************************************************************
*   tp_002
*******************************************************************************/
void
tp_002()
{
	int timeouts[3] =  {3, 5, 7};
	int timeout;
	int i;

	log_printf("#MLTC# {%s}{tp_002} Test verifies watchdog "
	           "set/get timeout function\n",
			 	tc_name);

	/* Open watchdog device file */
	wd_fd = open(wd_dev_name, O_WRONLY);
	if (wd_fd < 0){
		log_printf("#MLTC# {%s}{tp_002} Cannot open watchdog device\n", tc_name);
		FAIL = 1;
		return;
	}

	for(i=0; i < sizeof(timeouts) / sizeof(timeouts[0]); i++){
		if (ioctl(wd_fd, WDIOC_SETTIMEOUT, &timeouts[i]) < 0){
			log_printf("#MLTC# {%s}{tp_002} ioctl "
					   "WDIOC_SETTIMEOUT failed\n", tc_name);
			FAIL = 1;
			return;
		}
		if (ioctl(wd_fd, WDIOC_GETTIMEOUT, &timeout) < 0){
			log_printf("#MLTC# {%s}{tp_002} ioctl "
					   "WDIOC_GETTIMEOUT failed\n", tc_name);
			FAIL = 1;
			return;
		}
		if (timeout != timeouts[i]){
			log_printf("#MLTC# {%s}{tp_002} set timeout value(%d) != "
					   "get timeout value(%d) - FAIL\n",
					   tc_name, timeouts[i], timeout);
			FAIL = 1;
			return;
		}
		log_printf("#MLTC# {%s}{tp_002} set timeout value(%d) == "
					   "get timeout value(%d)\n",
					   tc_name, timeouts[i], timeout);
	}

	if (write(wd_fd, "V", 1) != 1){
		log_printf("#MLTC# {%s}{tp_002} write magic failed\n", tc_name);
		FAIL = 1;
		return;
	}

	if (close(wd_fd) <0){
		log_printf("#MLTC# {%s}{tp_002} close failed\n", tc_name);
		FAIL = 1;
		return;
	}

	log_printf("#MLTC# {%s}{tp_002} PASS\n", tc_name);
}

/*******************************************************************************
*   tp_003
*******************************************************************************/
void
tp_003()
{
	int timeout;

	log_printf("#MLTC# {%s}{tp_003} Test verifies watchdog "
	           "keepalive ioctl\n",
			 	tc_name);

	/* Open watchdog device file */
	wd_fd = open(wd_dev_name, O_WRONLY);
	if (wd_fd < 0){
		log_printf("#MLTC# {%s}{tp_003} Cannot open watchdog device\n", tc_name);
		FAIL = 1;
		return;
	}

	timeout = 4;
	if (ioctl(wd_fd, WDIOC_SETTIMEOUT, &timeout) < 0){
		log_printf("#MLTC# {%s}{tp_003} ioctl "
				   "WDIOC_SETTIMEOUT failed\n", tc_name);
		FAIL = 1;
		return;
	}
	
	sleep(timeout / 2);
	
	if (ioctl(wd_fd, WDIOC_KEEPALIVE, 0) < 0){
		log_printf("#MLTC# {%s}{tp_003} ioctl "
				   "WDIOC_KEEPALIVE failed\n", tc_name);
		FAIL = 1;
		return;
	}

	sleep(timeout / 2 + 1);

	if (write(wd_fd, "V", 1) != 1){
		log_printf("#MLTC# {%s}{tp_003} write magic failed\n", tc_name);
		FAIL = 1;
		return;
	}

	if (close(wd_fd) <0){
		log_printf("#MLTC# {%s}{tp_003} close failed\n", tc_name);
		FAIL = 1;
		return;
	}

	log_printf("#MLTC# {%s}{tp_003} PASS\n", tc_name);
}

/*******************************************************************************
*   tp_004
*******************************************************************************/
void
tp_004()
{
	int timeout;

	log_printf("#MLTC# {%s}{tp_004} Test verifies watchdog ping "
	           "by means of write() system call\n",
			 	tc_name);

	/* Open watchdog device file */
	wd_fd = open(wd_dev_name, O_WRONLY);
	if (wd_fd < 0){
		log_printf("#MLTC# {%s}{tp_004} Cannot open watchdog device\n", tc_name);
		FAIL = 1;
		return;
	}

	timeout = 4;
	if (ioctl(wd_fd, WDIOC_SETTIMEOUT, &timeout) < 0){
		log_printf("#MLTC# {%s}{tp_004} ioctl "
				   "WDIOC_SETTIMEOUT failed\n", tc_name);
		FAIL = 1;
		return;
	}
	
	sleep(timeout / 2);
	
	if (write(wd_fd, "ping", 4) != 4){
		log_printf("#MLTC# {%s}{tp_004} write() "
				   "system call failed\n", tc_name);
		FAIL = 1;
		return;
	}

	sleep(timeout / 2 + 1);

	if (write(wd_fd, "V", 1) != 1){
		log_printf("#MLTC# {%s}{tp_004} write magic failed\n", tc_name);
		FAIL = 1;
		return;
	}

	if (close(wd_fd) <0){
		log_printf("#MLTC# {%s}{tp_004} close failed\n", tc_name);
		FAIL = 1;
		return;
	}

	log_printf("#MLTC# {%s}{tp_004} PASS\n", tc_name);
}

/*******************************************************************************
*   tp_005
*******************************************************************************/
void
tp_005()
{
	int timeout;

	log_printf("#MLTC# {%s}{tp_005} Test verifies ability watchdog "
	           "to properly reboot system\n",
			 	tc_name);

	/* Open watchdog device file */
	wd_fd = open(wd_dev_name, O_WRONLY);
	if (wd_fd < 0){
		log_printf("#MLTC# {%s}{tp_005} Cannot open watchdog device\n", tc_name);
		FAIL = 1;
		return;
	}

	timeout = 5;
	if (ioctl(wd_fd, WDIOC_SETTIMEOUT, &timeout) < 0){
		log_printf("#MLTC# {%s}{tp_005} ioctl "
				   "WDIOC_SETTIMEOUT failed\n", tc_name);
		FAIL = 1;
		return;
	}

	log_printf("#MLTC# {%s}{tp_005} System would reboot after %d seconds\n",
			   tc_name, timeout);

	while(timeout){
		log_printf("#MLTC# {%s}{tp_005} Time to reboot : %d\n",
			   tc_name, timeout);
		sync();
		timeout--;
		sleep(1);
	}

	// Here system have to be rebooted
	
	if (close(wd_fd) <0){
		log_printf("#MLTC# {%s}{tp_005} close failed\n", tc_name);
		FAIL = 1;
		return;
	}

	log_printf("#MLTC# {%s}{tp_005} FAIL\n", tc_name);
	FAIL = 1;
}
