#!/bin/bash

set +xv

progname=$(basename $0)
cd $(dirname $0)

BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TMP_DIR=${TC_DIR}/tmp

tmpf=/tmp/_$progname_

GOSU=su_osltc

if [[ -z $BCONF ]]; then
	### bootconf_osltc returns bootloader config file:
        ### /boot/silo | /boot/grub/grub.conf
	BCONF=$(bootconf_osltc)
	[[ $? != 0 ]] && exit 1
fi

echo "$progname: HELLO: <comments>" >${TMP_DIR}/selftest_stdout

RES_TEST=0

if [[ $RES_TEST == 0 ]]; then
	echo "$progname: PASS" >>${TMP_DIR}/selftest_stdout
	>${TMP_DIR}/PASS
fi
if [[ $RES_TEST != 0 ]]; then
	echo "$progname: FAIL" >>${TMP_DIR}/selftest_stdout
	>${TMP_DIR}/FAIL
fi

### Restore system files:
### /etc/passwd, /etc/inittab, /boot/silo.conf (/boot/grub/grub.conf)

SysCleanUp()
{	
	cmd="$GOSU cp ${TMP_DIR}/boot.conf $BCONF"
	$cmd >$tmpf 2>&1
	ret=$?
	if [[ $ret != 0 ]]; then
		echo "$progname: $cmd - returned $ret"
		cat $tmpf
		return 1
	else
		echo "$progname: $cmd - passed"
	fi

	cmd="$GOSU cp ${TMP_DIR}/passwd /etc/passwd"
	$cmd >$tmpf 2>&1
	ret=$?
	if [[ $ret != 0 ]]; then
		echo "$progname: $cmd - returned $ret"
		cat $tmpf
		return 1
	else
		echo "$progname: $cmd - passed"
	fi

	cmd="$GOSU cp ${TMP_DIR}/inittab /etc/inittab"
	$cmd >$tmpf 2>&1
	ret=$?
	if [[ $ret != 0 ]]; then
		echo "$progname: $cmd - returned $ret"
		cat $tmpf
		return 1
	else
		echo "$progname: $cmd - passed"
	fi
	return 0
}

SysCleanUp >${TMP_DIR}/SYS_CLEANUP
ret=$?

rm -f $tmpf

[[ $ret == 0 ]] && $GOSU /sbin/reboot && exit 0

### FATAL ERROR: testing process completes

cat ${TMP_DIR}/SYS_CLEANUP
exit 1


