#!/bin/bash

set +xv

progname=$(basename $0)
cd $(dirname $0)

BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TMP_DIR=${TC_DIR}/tmp
CFG_DIR=${TC_DIR}/cfg
ENV_FILE=${CFG_DIR}/env

tmpf=/tmp/_$progname_

echo "$progname: HELLO: <comments>"

if [[ -z $BTYPE ]]; then
	### boottype_osltc returns bootloader type: silo|grub"
	echo "$progname: get BootLoader type"
	BTYPE=$(boottype_osltc)
	[[ $? != 0 ]] && exit 1
fi
echo "$progname: BootLoader type: $BTYPE"
if [[ -z $BCONF ]]; then
	### bootconf_osltc returns bootloader config file:
        ### /boot/silo | /boot/grub/grub.conf
	echo "$progname: get BootLoader config file name"
	BCONF=$(bootconf_osltc)
	[[ $? != 0 ]] && exit 1
fi
echo "$progname: BootLoader config file: $BCONF"

mkdir -p -m 0777 $TMP_DIR
rm -f ${TMP_DIR}/PASS ${TMP_DIR}/FAIL
rm -f >${TMP_DIR}/SYS_CLEANUP
rm -f ${TMP_DIR}/selftest_stdout

rm -f ${TMP_DIR}/boot.conf.test ${TMP_DIR}/passwd.test ${TMP_DIR}/inittab.test

if [[ ! -f ${TMP_DIR}/boot.conf ]]; then
	cmd="su_osltc cp $BCONF ${TMP_DIR}/boot.conf"
	echo "$progname: $cmd"
	$cmd
	[[ $? != 0 ]] && exit 1
	su_osltc chmod 666 ${TMP_DIR}/boot.conf
fi
if [[ ! -f ${TMP_DIR}/passwd ]]; then
	cmd="su_osltc cp /etc/passwd ${TMP_DIR}/passwd"
	echo "$progname: $cmd"
	$cmd
	[[ $? != 0 ]] && exit 1
	su_osltc chmod 666 ${TMP_DIR}/passwd
fi
if [[ ! -f ${TMP_DIR}/inittab ]]; then
	cmd="su_osltc cp /etc/inittab ${TMP_DIR}/inittab"
	echo "$progname: $cmd"
	$cmd
	[[ $? != 0 ]] && exit 1
	su_osltc chmod 666 ${TMP_DIR}/inittab
fi

### create files for run 'selftest'
### (passwd.test, inittab.test, and boot.conf.test)
### 'selftest' will be run after /sbin/reboot (see ../scen)

user=$(whoami)

echo "$progname: Create ${TMP_DIR}/boot.conf.test"
mk_bootconf_osltc -b $BTYPE -i ${TMP_DIR}/boot.conf -o ${TMP_DIR}/boot.conf.test \
	-l selftest -a "init=/sbin/init startuser=$user 3"
#	-l selftest -a "console=ttyS0,9600n8 serial=0,9600n8  init=/sbin/init startuser=$user 3"
[[ $? != 0 ]] && exit 1

echo "$progname: Create ${TMP_DIR}/passwd.test"
mk_passwd_osltc -i ${TMP_DIR}/passwd -o ${TMP_DIR}/passwd.test -p ${BIN_DIR}/selftest
[[ $? != 0 ]] && exit 1

echo "$progname: Create ${TMP_DIR}/inittab.test"
cat ${TMP_DIR}/inittab | grep /mcst/sbin/getty >/dev/null 2>&1
ret=$?
if [[ $ret == 0 ]]; then
	cp ${TMP_DIR}/inittab ${TMP_DIR}/inittab.test
else
	mk_inittab_osltc -i ${TMP_DIR}/inittab -o ${TMP_DIR}/inittab.test -p /mcst/sbin/getty
	[[ $? != 0 ]] && exit 1
fi
cp ${TMP_DIR}/inittab ${TMP_DIR}/inittab.test

### copy created passwd.test, inittab.test, and boot.conf.test files
### into system files

cmd="su_osltc cp ${TMP_DIR}/boot.conf.test $BCONF"
echo "$progname: $cmd"
$cmd
[[ $? != 0 ]] && exit 1

cmd="su_osltc cp ${TMP_DIR}/passwd.test /etc/passwd"
echo "$progname: $cmd"
$cmd
if [[ $? != 0 ]]; then
	su_osltc cp ${TMP_DIR}/boot.conf $BCONF
	exit 1
fi

cmd="su_osltc cp ${TMP_DIR}/inittab.test /etc/inittab"
echo "$progname: $cmd"
$cmd
if [[ $? != 0 ]]; then
	su_osltc cp ${TMP_DIR}/boot.conf $BCONF
	su_osltc cp ${TMP_DIR}/passwd /etc/passwd
	exit 1
fi

su_osltc cat $BCONF | grep "^default=selftest"
ret=$?
if [[ $ret != 0 ]]; then
	echo "$progname: Incorrect $BCONF"
	exit 1
fi

exit 0
