#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include <sched.h>
#include <mcst_rt.h>

static int verbouse = 0;
#define printf_v(args...)       \
	if (verbouse){          \
		printf(args);   \
		fflush(stdout); \
	}

#define PROG_OPT        "vh"

// Usage
void usage(char *name) {
	printf("%s usage :\n", name);
	printf("-h         show help\n");
	printf("-v         verbouse\n");
}

long	cpusnum = 0;        // count of active processors
int	*cpus_id = NULL;    // array of active processors id

// Bind process to CPU
int bind_to_cpu(pid_t pid, int cpu){
	cpu_set_t mask;
	CPU_ZERO(&mask);
	CPU_SET(cpu, &mask);
	return sched_setaffinity(pid, sizeof(cpu_set_t), &mask);
}

int worker_proc(int rfd, int wfd)
{
        int mes, pr;

       while(1) {
                if (read(rfd, &mes, sizeof(mes)) < sizeof(mes)) {
			break;
		}
                pr = el_get_cpu_id();
                printf_v("WORKER : expected_cpu = %d : real_cpu = %d\n", mes, pr);
                if (write(wfd, &pr, sizeof(pr)) != sizeof(pr)) {
                        break;
                }
       }
	return 0;
}

int main(int argc, char **argv)
{
	pid_t                worker;             // id of worker process
	long                 cpusmask = 0;       // mask of ative processors
	int                  pipe_mw[2];         // using to organize pipes manager-worker
	int                  pipe_wm[2];         // and worker-manager
	int                  mes;                // using to store message from pipe
	int                  opt;
	int                  i, j, p;
	int                  retval = 0;
	int                  fail = 0;

        while ((opt = getopt(argc, argv, PROG_OPT)) != EOF) {
                switch (opt) {
		case 'v' :
			verbouse = 1;
			break;
		case 'h' :
			usage(argv[0]);
			return 0;
		default:
			usage(argv[0]);
			return 1;
		}
	}
	
	// Create pipe
	retval = pipe(pipe_mw);
	if ( retval < 0 ){
		perror("pipe() ");
		printf("%s FAIL\n", argv[0]);
		return 1;
	}
	retval = pipe(pipe_wm);
	if ( retval < 0 ){
		perror("pipe() ");
		printf("%s FAIL\n", argv[0]);
		return 1;
	}

	// Create worker process
	worker = fork();
	if ( worker < 0 ){
		perror("fork() ");
		printf("%s FAIL\n", argv[0]);
		return 1;
	}
        if (worker == 0) {
                exit(worker_proc(pipe_mw[0], pipe_wm[1]));
        }

	printf_v("MANAGER : worker = %d\n", worker);

	// Init array of cpu ids
	cpusnum = el_get_cpus_num();
	cpusmask = el_get_cpus_mask();
	cpus_id = calloc(cpusnum, sizeof(int));
	i = j = 0;
	while (i < cpusnum) {
		while (! (cpusmask & 1)) { cpusmask = cpusmask >> 1; j++; }
		cpus_id[i] = j;
		i++;
		cpusmask = cpusmask >> 1;
		j++;
	}

        //  General test loop
        for( p = 0; p < cpusnum; p++ ) {
            // Bind manager to target cpu
            if (bind_to_cpu(0, cpus_id[p])) {
                perror("Bind manager to target cpu");
		fail = 1;
                goto out;
            }
            printf_v("MANAGER: bound to cpu %d\n", cpus_id[p]);

            // Bind worker to target cpu
            if (bind_to_cpu(worker, cpus_id[p])) {
                perror("Bind worker to target cpu");
		fail = 1;
                goto out;
            }
            printf_v("MANAGER : worker expected_cpu = %d\n", cpus_id[p]);
            
            if (write(pipe_mw[1], &cpus_id[p], sizeof(cpus_id[0])) != sizeof(cpus_id[0])) {
                fprintf(stderr, "Manager:can't write to pipe\n");
		fail = 1;
		goto out;
	    }
            if (read(pipe_wm[0], &mes, sizeof(mes)) != sizeof(mes)) {
	        fprintf(stderr, "Manager:can't read pipe\n");
		fail = 1;
		goto out;
	    }
                       
            printf_v("MANAGER : worker real_cpu = %d %s\n\n",
                      mes, (mes ==cpus_id[p]) ? "" :  "cpu mismatched !!!!!!");

            if ( mes != cpus_id[p] ) {
                fail = 1;
                goto out;
            }
         }

out:
        if (worker) {
                kill(worker, SIGKILL);
        }

	free(cpus_id);

	if (!fail )
		printf("\n%s : PASS\n", argv[0]);
	else
		printf("\n%s : FAIL\n", argv[0]);

	exit(fail);
}
