#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

VERBOSE=0


###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}

verboseEcho()
{
	[ $VERBOSE == 1 ] && doEcho $*
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname [-hv]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":hv" optname
do
	case "$optname" in
	"h")
		usage
	;;
	"v")
		VERBOSE=1
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# Run test 
###############################################################################
doEcho "debian version test case start at $(date)"

# Skip test on OSElbrus distr
[ -f /etc/init.d/mcst.start ] && exit $(skip_osltc) 

# Debian version
if [ ! -f /etc/debian_version ];
then
	doEcho "File /etc/debian_version does not exist"
	exit 1
fi 
DEBVER=$(cat /etc/debian_version)
doEcho "Debian version = $DEBVER"

# Mcst version
#if [ ! -f /etc/mcst_version ];
#then
#	doEcho "File /etc/mcst_version does not exist"
#	exit 1
#fi 
#MCSTVER=$(cat /etc/mcst_version)
#doEcho "Mcst version = $MCSTVER"

# Compare
if [  "8.0" = "$(echo -e "8.0\n$DEBVER" | sort -V | head -n1)" ];
then
	doEcho "Debian version is not less then 8.0"
else
	doEcho "Debian version is less then 8.0"
	exit 1
fi

#if [  "$DEBVER" = "$(echo -e "$DEBVER\n$MCSTVER" | sort -V | head -n1)" ];
#then
#	doEcho "Mcst version is not less then debian version"
#else
#	doEcho "Mcst version is less then debian version"
#	exit 1
#fi

###############################################################################
# Finish test
###############################################################################
doEcho "debian version test case stop at $(date)"

exit 0 
