#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

VERBOSE=0


###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}

verboseEcho()
{
	[ $VERBOSE == 1 ] && doEcho $*
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname [-hv]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":hv" optname
do
	case "$optname" in
	"h")
		usage
	;;
	"v")
		VERBOSE=1
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# Run test 
###############################################################################
doEcho "iohub test case start at $(date)"

IOHUBCONF="CONFIG_IOHUB_GPIO=y"

# Check config
if zcat /proc/config.gz | grep -q "$IOHUBCONF";
then
	doEcho "Kernel is built with CONFIG_IOHUB_GPIO=y"
else
	doEcho "Kernel is built without CONFIG_IOHUB_GPIO=y"
	exit 1
fi

###############################################################################
# Finish test
###############################################################################
doEcho "iohub test case stop at $(date)"

exit 0 
