#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

VERBOSE=0


###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}

verboseEcho()
{
	[ $VERBOSE == 1 ] && doEcho $*
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname [-hv]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":hv" optname
do
	case "$optname" in
	"h")
		usage
	;;
	"v")
		VERBOSE=1
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# Run test 
###############################################################################
doEcho "kernel version test case start at $(date)"

# kernel version
KVER=$(uname -r)
KVER=${KVER%%-*}
doEcho "Kernel version = $KVER"

# Compare
if [  "5.4" = "$(echo -e "5.4\n$KVER" | sort -V | head -n1)" ];
then
	doEcho "Kernel version is not less then 5.4"
else
	doEcho "Kernel version is less then 5.4"
	exit 1
fi

###############################################################################
# Finish test
###############################################################################
doEcho "kernel version test case stop at $(date)"

exit 0 
