#!/bin/bash

progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

SIGSEGV_RET=$(echo "128 + $(kill -l SIGSEGV)" | bc)
SIGILL_RET=$(echo "128 + $(kill -l SIGILL)" | bc)

###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname [-h]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":g:i:o:r:s:c:d:u:p:hv" optname
do
	case "$optname" in
	"h")
		usage
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# COMPILE TEST
###############################################################################
doEcho "Compile test"

cat > test.c << EOF
#include <stdlib.h>
int main()
{
    int ret = 0;

    if( rand() /2 )
        ret=*((int*)NULL);
    else
        ret=*((int*)(NULL+8));

    return ret;
}
EOF

cat test.c

ret=PASS
gcc -O0 test.c -o test.O0 || ret=FAIL
[ "$ret" == "FAIL" ] && doEcho "FAIL" && exit 1

ret=PASS
gcc -O2 test.c -o test.O2 || ret=FAIL
[ "$ret" == "FAIL" ] && doEcho "FAIL" && exit 1


###############################################################################
# RUN TEST
###############################################################################
doEcho "Run test"

sigdebug=$(cat /proc/sys/debug/sigdebug)
echo 1 > /proc/sys/debug/sigdebug

dmesg -c > /dev/null 2>&1

doEcho "Run test in origin mode"
./test.O0
[ "$?" != "$SIGSEGV_RET" ] && doEcho "FAIL" && exit 1
doEcho "Get SIGSEGV"

doEcho "Run test in sm mode"
./test.O2
[ "$?" != "$SIGILL_RET" ] && doEcho "FAIL" && exit 1
doEcho "Get SIGILL"

dmesg

echo $sigdebug > /proc/sys/debug/sigdebug

doEcho "PASS"
exit 0
