#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

VERBOSE=0

RET=0

PAUSE=10

SIFACE=""
CIFACE=""

###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}

verboseEcho()
{
	[ $VERBOSE == 1 ] && doEcho $*
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname -s serv_iface -c client_iface [-p pause] [-hv]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":s:c:p:hv" optname
do
	case "$optname" in
	"s")
		SIFACE="$OPTARG"
	;;
	"c")
		CIFACE="$OPTARG"
	;;
	"p")
		PAUSE="$OPTARG"
	;;
	"h")
		usage
	;;
	"v")
		VERBOSE=1
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done

[ "$SIFACE" == "" -o "$CIFACE" == "" ] && usage 

###############################################################################
# Run test 
###############################################################################
doEcho "pppoe test case start at $(date)"

cp /etc/ppp/pppoe-server-options /tmp
cp /etc/ppp/chap-secrets /tmp

for IFACE in $SIFACE $CIFACE;
do
	ifconfig $IFACE up > /dev/null 2>&1
	addr="$(ip addr show $IFACE | grep "inet " | awk '{print $2}')"
	[ ! -z "$addr" ] && ip addr del $addr dev $IFACE
done

ifconfig lo up

cat > /etc/ppp/pppoe-server-options  << EOF
auth
require-chap
ms-dns 81.30.199.5
ms-dns 81.30.199.94
noipdefault
noipx
nodefaultroute
noproxyarp
netmask 255.255.255.255
logfile /var/log/pppoe-server.log
EOF

echo "testu * testp 10.10.10.2" > /etc/ppp/chap-secrets

cat > /etc/ppp/peers/pppoe_tc  << EOF
user testu
plugin rp-pppoe.so
noipdefault
usepeerdns
persist
noauth
unit 2
$CIFACE
EOF

pppoe-server -I $SIFACE -L 10.10.10.1
pppoe-discovery -I $CIFACE
pon pppoe_tc

echo 1 > /proc/sys/net/ipv4/conf/all/accept_local

sleep $PAUSE

ping 10.10.10.1 -I ppp2 -c 1 > /dev/null 2>&1
[ "$?" != "0" ] && RET=1 && echo FAIL || echo PASS

poff -a
killall pppoe-server

cp /tmp/pppoe-server-options  /etc/ppp/ 
cp /tmp/chap-secrets  /etc/ppp/ 
rm /etc/ppp/peers/pppoe_tc
 
###############################################################################
# Finish test
###############################################################################
doEcho "pppoe test case stop at $(date)"

exit $RET
