#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

VERBOSE=0

RET=0

PAUSE=3

###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}

verboseEcho()
{
	[ $VERBOSE == 1 ] && doEcho $*
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname [-p pause] [-hv]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":s:c:p:hv" optname
do
	case "$optname" in
	"p")
		PAUSE="$OPTARG"
	;;
	"h")
		usage
	;;
	"v")
		VERBOSE=1
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# Run test 
###############################################################################
doEcho "pptp test case start at $(date)"

# Skip test on OSElbrus distr
[ -f /etc/init.d/mcst.start ] && exit $(skip_osltc) 

cp /etc/pptpd.conf /tmp
cp /etc/ppp/pptpd-options /tmp
cp /etc/ppp/chap-secrets /tmp
cp /etc/ppp/options.pptp /tmp	

ifconfig lo up

cat > /etc/pptpd.conf << EOF
option /etc/ppp/pptpd-options
logwtmp
localip 10.10.10.1
remoteip 10.10.10.10-20
EOF

cat > /etc/ppp/pptpd-options << EOF
name pptpd
refuse-pap
refuse-chap
refuse-mschap
require-mschap-v2
require-mppe-128
proxyarp
nodefaultroute
lock
nobsdcomp
novj
novjccomp
nologfd
EOF

echo "testu pptpd testp *" > /etc/ppp/chap-secrets

cat > /etc/ppp/options.pptp << EOF
lock
noauth
refuse-pap
refuse-eap
refuse-chap
refuse-mschap
nobsdcomp
nodeflate
require-mppe-128
EOF

cat > /etc/ppp/peers/pptp_tc  << EOF
pty "pptp 127.0.0.1 --nolaunchpppd"
name testu
remotename pptpd
file /etc/ppp/options.pptp
ipparam vpn
EOF

echo 1 > /proc/sys/net/ipv4/conf/all/accept_local

systemctl restart pptpd
sleep $PAUSE

for i in $(seq 1 10);
do
	pon pptp_tc;
	sleep $PAUSE;
	(ip link show | grep -q ppp0) && break; 
done

ping 10.10.10.1 -I ppp0 -c 1 > /dev/null 2>&1
[ "$?" != "0" ] && RET=1 && echo FAIL || echo PASS

poff -a
systemctl stop pptpd

cp /tmp/pptpd.conf /etc
cp /tmp/pptpd-options /etc/ppp
cp /tmp/chap-secrets /etc/ppp
cp /tmp/options.pptp /etc/ppp	
rm /etc/ppp/peers/pptp_tc
 
###############################################################################
# Finish test
###############################################################################
doEcho "pptp test case stop at $(date)"

exit $RET
