#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

VERBOSE=0

RET=0

PAUSE=3

###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}

verboseEcho()
{
	[ $VERBOSE == 1 ] && doEcho $*
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname [-p pause] [-hv]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":p:hv" optname
do
	case "$optname" in
	"p")
		PAUSE="$OPTARG"
	;;
	"h")
		usage
	;;
	"v")
		VERBOSE=1
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done


###############################################################################
# Restart service
###############################################################################
service_restart()
{
	if [ -f /etc/issue ]; then
		# ELBRUSD
		systemctl restart $1
	else
		#OSELBRUS
		service $1 restart
	fi
}


###############################################################################
# Run test 
###############################################################################
doEcho "snmp test case start at $(date)"

cp /etc/snmp/snmpd.conf /tmp/

echo "rocommunity public  localhost" > /etc/snmp/snmpd.conf
service_restart snmpd > /dev/null 2>&1

sleep $PAUSE

ic="$(snmpwalk -v 1 -c public 127.0.0.1  .1.3.6.1.2.1.2.1.0)"
ic="${ic##* }"

ic2="$(ip link show | grep link | wc -l)"

[ "$ic" != "$ic2" ] && RET=1 && echo FAIL || echo PASS

cp /tmp/snmpd.conf /etc/snmp
service_restart snmpd > /dev/null 2>&1

###############################################################################
# Finish test
###############################################################################
doEcho "snmp test case stop at $(date)"

exit $RET
