#!/bin/bash

###############################################################################
# Variables
###############################################################################
progname=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

VERBOSE=0

RET=0

PAUSE=3
MAGIC="===XINETD TEST CASE==="

###############################################################################
# Print message
###############################################################################
doEcho()
{
	echo "=== ${progname} === $*"
}

verboseEcho()
{
	[ $VERBOSE == 1 ] && doEcho $*
}


###############################################################################
# Show usage and exit
###############################################################################
usage()
{
	doEcho "Usage: $progname [-p pause] [-hv]"
	exit 1;
}


###############################################################################
# Parsing input parameters and set variables
###############################################################################
while getopts ":p:hv" optname
do
	case "$optname" in
	"p")
		PAUSE="$OPTARG"
	;;
	"h")
		usage
	;;
	"v")
		VERBOSE=1
	;;
	"?")
		doEcho "Unknown option $OPTARG"
		usage
	;;
	":")
		doEcho "No argument value for option $OPTARG"
		usage
	;;
	*)
		usage
	;;
	esac
done

test_echo()
{
	sleep $PAUSE

	exec 3<>/dev/tcp/127.0.0.1/7
	(echo -e "$MAGIC\n" >&3)
	REPLY=$(head -n 1 <&3)
	exec 3<&-
	exec 3>&-

	[ "$REPLY" != "$MAGIC" ] && return 1

	return 0
}


###############################################################################
# Restart service
###############################################################################
service_restart()
{
	if [ -f /etc/issue ]; then
		# ELBRUSD
		systemctl restart $1
	else
		#OSELBRUS
		service $1 restart
	fi
}


###############################################################################
# Run test 
###############################################################################
doEcho "xinetd test case start at $(date)"

# Skip test on OSElbrus distr
[ -f /etc/init.d/mcst.start ] && exit $(skip_osltc)

cp /etc/xinetd.d/echo /tmp

doEcho "test echo service"
sed -i '/disable/d' /etc/xinetd.d/echo
sed -i 's/{/{\n\tdisable = no/g' /etc/xinetd.d/echo
service_restart xinetd > /dev/null 2>&1
ret=0
test_echo || ret=1
[ "$ret" == "1" ] && RET=1 && echo FAIL || echo PASS

doEcho "test echo service with access_times"
AHOUR=$(date +%H)
AHOUR=$(echo "($AHOUR+2) % 24" | bc)
ATIMES="$AHOUR:00-$AHOUR:59"
sed -i '/access_times/d' /etc/xinetd.d/echo
sed -i 's/{/{\n\taccess_times = $ATIMES/g' /etc/xinetd.d/echo
service_restart xinetd > /dev/null 2>&1
ret=0
test_echo && ret=1
[ "$ret" == "1" ] && RET=1 && echo FAIL || echo PASS

cp /tmp/echo /etc/xinetd.d/echo
service_restart xinetd > /dev/null 2>&1

###############################################################################
# Finish test
###############################################################################
doEcho "xinetd test case stop at $(date)"

exit $RET
