#!/bin/sh

set +xv

progname=$(basename $0)
cd $(dirname $0)

BIN_DIR=$(pwd)
TC_DIR=$(dirname $BIN_DIR)
TMP_DIR=${TC_DIR}/tmp
CFG_DIR=${TC_DIR}/cfg
ENV_FILE=${CFG_DIR}/env

tmpf=/tmp/_${progname}_tmpf

uid=$(id -u)
if [[ $uid == 0 && 1 == 0 ]]; then
	echo "Test should be run as not super user"
	exit 1
fi

verbose=0

typeset -i RET=0

DoEcho()
{
	echo "$progname: $*"
}
DoEchoVer()
{
	[[ $verbose == 1 ]] && DoEcho "$*"
}
DoCleanUp()
{
	rm -f $tmpf
#	mv core* ${TMP_DIR} 1>&2  2>/dev/null
	rm dump* ${TMP_DIR} 1>&2  2>/dev/null
}
DoExit()
{
	DoCleanUp
	[[ ! -z $2 ]] && DoEcho "$2"
	[[ $1 == 0 ]] && DoEcho PASS
	[[ $1 > 0 ]] && DoEcho FAIL
	exit $1
}
################################################################################
usage()
{
	echo " usage: $progname [-v]"
	echo
	exit 1
}
OPTIONS="v"
while getopts $OPTIONS option
do
	case $option in
		v) verbose=1; opt_v="-v";;
		*) echo "$prog: illegal option -- $option"; usage; exit 1;;
	esac
done
################################################################################
coredump()
{
	DoEcho "Сброс core dump программы пользовавателя в любом месте"
	rm core*  1>&2  2>/dev/null
	ulimit -c 99999
	./test_dump 1>/dev/null 2>&1
	if [[ $? -eq 0 ]]; then
		DoEcho "./test_dump - failed  $? "
		(( RET++ ))
	else
		DoEchoVer "./test_dump - passed"
	fi

	DoEchoVer "core dump файлы смотрим с помощью gdb (gdb ./test_dump core<pid>)"
	DoEcho "Просмотр core_dump файлов при помощи gdb"
	corefile=core*
	for file in $corefile ; do
		#gdb -x GG -c $file | grep "main" | wc -l 1>/dev/null 2>&1
		gdb -x GG -c $file </dev/null 1>$tmpf 2>&1
		ret=$?
		if [[ $ret != 0 ]]; then
			DoEcho "gdb -x GG -c $file - failed"
			(( RET++ ))
			continue
		else
			DoEchoVer "gdb -x GG -c $file - passed"
		fi
		cat $tmpf | grep "main" | wc -l 1>/dev/null 2>&1
		ret=$?
		if [[ $ret != 0 ]]; then
			DoEcho "TEST CORE_DUMP c $file - failed"
			(( RET++ ))
		else
			DoEchoVer "TEST CORE_DUMP c $file - passed"
		fi
	done
}
################################################################################
rm -rf $TMP_DIR
mkdir -m 0777 $TMP_DIR

rm core*  1>/dev/null 2>&1
rm dump* 1>/dev/null 2>&1

# these tests may be work on sparc too !!!

PARAM="-10 -22 -k10 "

DoEcho "TEST RUN"

if [[ $(uname -m) == e2k && 1 == 0 ]] ;then
	# only for  E2K   
	> /var/log/messages
	sync
	DoEcho "Определение  времени закрытых внешних прерываний и non-preemptable"
	#./trace-it "./prt $PARAM " 4  
	sync
	./prt $PARAM 1>/dev/null 2>&1
	if [[ $? -ne 0 ]]; then
		DoEcho "./prt $PARAM -failed"
		(( RET++ ))
	else
		DoEcho "./prt $PARAM - passed"
	fi
	#cat  /var/log/messages | sed %s/^.*kernel:// |sed %s/\^I\\|grep -A 400 ==============  > RESULT_0
	cat  /var/log/messages | sed -e "s/^.*kernel://;s/\^I^I/\t\t/;s/\^I^I/\t\t/;s/\^I/\t/" | grep "disabled interrupts" >${TMP_DIR}/RESULT_0
	cat  /var/log/messages | sed -e "s/^.*kernel://;s/\^I^I/\t\t/;s/\^I^I/\t\t/;s/\^I/\t/" | grep "preempt_count  max time" >>${TMP_DIR}/RESULT_0
	if [[ $? -ne 0 ]]; then
		DoEcho "kernel compiled  w/o CONFIG_PROFILING - failed"
		(( RET++ ))
	else
		DoEcho "pезультат: ../tmp/RESULT_0"
	fi         
fi

VER33=$(uname -r| cut -d '.' -f 3)

if [[  $VER33 == 33* ]]; then
	coredump
	DoExit $RET

	/bin/mount -t debugfs nodev /sys/kernel/debug 2> /dev/null
	cat /sys/kernel/debug/tracing/trace 1>/dev/null 2>&1
else
 # 2.6.14
 cat /proc/latency_trace  1>/dev/null 2>&1
fi

if [[ $? -ne 0 ]]; then
	DoExit 1 "KERNEL должен быть оттранслирован с режимом трассировки"
fi
rm dump* 1>/dev/null 2>&1

DoEcho "ТРАСCИРОВКА ОС и пользователя (сброс дампа трассировки)" 
if [[  $VER33 == 33* ]]; then
#        echo nop >  /sys/kernel/debug/tracing/current_tracer
        echo 100 > /sys/kernel/debug/tracing/buffer_size_kb  
        echo 1000 > /sys/kernel/debug/tracing/buffer_size_kb  
        echo sched_switch >  /sys/kernel/debug/tracing/current_tracer
        echo 1 >  /sys/kernel/debug/tracing/tracing_on
        echo 1 > /sys/kernel/debug/tracing/tracing_enabled
       ./prt_dump  $PARAM 1>${TMP_DIR}/RESULT_prt_dump 2>&1
        # echo res of  prt_dump = $?      
        echo 0 >  /sys/kernel/debug/tracing/tracing_on
        echo 0 > /sys/kernel/debug/tracing/tracing_enabled
        cat /sys/kernel/debug/tracing/trace  1>${TMP_DIR}/RESULT_3 2>&1
#       cat ${TMP_DIR}/RESULT_3
else  
        ./prt_dump  $PARAM 1>${TMP_DIR}/RESULT_prt_dump 2>&1
fi

if [[ $? -ne 0 ]]; then
	DoEcho "./prt_dump - failed"
	cat ${TMP_DIR}/RESULT_prt_dump
	exit 1
	(( RET++ ))
else
	DoEchoVer "./prt_dump - passed"
fi

DoEcho "Полная печать  дампа трассировки"
print-trace -d dump* -f >${TMP_DIR}/RESULT_1
if [[  $VER33 == 33* ]]; then
#       echo 001 ${TMP_DIR}/RESULT_3
        cat ${TMP_DIR}/RESULT_3 | tail +11 >> ${TMP_DIR}/RESULT_1
else
        if [[ $? -ne 0 ]]; then
	        DoEcho "print-trace -f - failed"
	        DoEcho "pезультат: ../tmp/RESULT_1"
	        cat ${TMP_DIR}/RESULT_1
	        exit 1
	        (( RET++ ))
        else
	        DoEchoVer "print-trace -f - passed"
	        DoEchoVer "pезультат: ../tmp/RESULT_1"
        fi
fi


DoEcho "Печать  дампа только пользователя"
if [[  $VER33 == 33* ]]; then
        print-trace -d dump* -f  prt_dump >${TMP_DIR}/RESULT_2
else
        print-trace -d dump* -n kernel >${TMP_DIR}/RESULT_2
fi
#echo RES $?
if [[ $? -ne 0 ]]; then
	DoEcho "print-trace -n kernel - failed (result=$?) "
	DoEcho "pезультат: ../tmp/RESULT_2 ?"
	cat ${TMP_DIR}/RESULT_2
	exit 1
	(( RET++ ))
else
	DoEchoVer "print-trace -n kernel - passed"
	DoEchoVer "pезультат: ../tmp/RESULT_2"
fi


DoEcho "Печать  дампа только ОС"
if [[  $VER33 != 33* ]]; then
        print-trace -d dump* -p kernel >${TMP_DIR}/RESULT_3
fi        
if [[ $? -ne 0 ]]; then
	DoEcho "print-trace -p kernel - failed"
	DoEcho "pезультат: ../tmp/RESULT_3"
	cat ${TMP_DIR}/RESULT_3
	exit 1
	(( RET++ ))
else
	DoEchoVer "print-trace -p kernel - passed"
	DoEchoVer "pезультат: ../tmp/RESULT_3"
fi

coredump

DoExit $RET



