#!/bin/bash

PROGNAME=$(basename $0)

CWD=$(dirname $0)
cd $CWD
CWD=$PWD

LOG="$CWD/$PROGNAME.log"
echo "" > $LOG

TMPDIR="$CWD/tmp"
rm -rf $TMPDIR
mkdir $TMPDIR

PROG="main"

CORE_PATTERN=$(cat /proc/sys/kernel/core_pattern)
cleanup()
{
	echo "$CORE_PATTERN" > /proc/sys/kernel/core_pattern
}
trap cleanup EXIT

echo core > /proc/sys/kernel/core_pattern

###############################################################################
# COMPILE TEST
###############################################################################
echo -n "Compile prog : "

cat > $TMPDIR/$PROG.c << EOF
int COUNT = 0;

void f1() { int *p = ((int*)-1); *p = 0xFFFFFFFF; }
void f2() { COUNT++; f1(); }
void f3() { COUNT++; f2(); }

int main()
{	
	f3();
	return 0;
}
EOF
cat $TMPDIR/$PROG.c >> $LOG

gcc -g $TMPDIR/$PROG.c -o $TMPDIR/$PROG
if [ $? != 0 ]; then 
	echo "FAIL"
	exit 1
else
	echo "PASS"
fi

###############################################################################
# GET CORE ON CRASH
###############################################################################
echo -n "Get core on SIGSEGV : "

ulimit -c 99999
rm -f ./core*
$TMPDIR/$PROG > /dev/null 2>&1
core=`ls | grep core | head -n 1`
mv ./$core $TMPDIR/core.crash
if [ $? != 0 ]; then
	echo "FAIL"
	exit 1
else
	echo "PASS"
fi
rm -f ./core*

###############################################################################
# GET CORE BY COMMAND
###############################################################################
echo -n "Save core with gdb command : "

cat > $TMPDIR/s1 << EOF
file $TMPDIR/$PROG
break f2
run
gcore
q
EOF

echo "#" >> $LOG
echo "# Save core with gdb command" >> $LOG
echo "#" >> $LOG
gdb -x $TMPDIR/s1 -q >> $LOG 2>&1
if [ $? != 0 ]; then 
	echo "FAIL"
	exit 1
else
	echo "PASS"
fi
echo "" >> $LOG
core=`ls | grep core | head -n 1`
mv ./$core $TMPDIR/core.cmd
rm -f ./core*

###############################################################################
# ANALYZE CORE
###############################################################################
echo -n "Analyze core : "

cat > $TMPDIR/s2 << EOF
file $TMPDIR/$PROG
bt
print COUNT
q
EOF

echo "#" >> $LOG
echo "# Analyze crash and saved cores" >> $LOG
echo "#" >> $LOG
for core in core.crash core.cmd; do
	gdb -x $TMPDIR/s2 -c $TMPDIR/$core -q >> $LOG 2>&1
	if [ $? != 0 ]; then 
		echo "FAIL"
		exit 1
	fi
	echo "" >> $LOG
done

echo "PASS"
exit 0
